/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.action.support;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.ws.rs.Path;

import org.loom.action.AbstractAction;
import org.loom.annotation.Cache;
import org.loom.annotation.Event;
import org.loom.annotation.SSLPolicy;
import org.loom.annotation.SSLPolicyType;
import org.loom.config.Config;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.resolution.AbstractHttpResolution;
import org.loom.resolution.HttpErrorResolution;
import org.loom.resolution.InputStreamResolution;
import org.loom.resolution.Resolution;
import org.loom.resolution.StringResolution;
import org.loom.resources.WebResourceData;
import org.loom.resources.WebResourcesRepository;
import org.loom.servlet.CacheControl;
import org.loom.servlet.names.ContentTypeNames;
import org.loom.servlet.names.RequestHeaderNames;
import org.loom.servlet.names.ResponseHeaderNames;


/**
 * Retrieves a static resource, such as a javascript/css file.
 * It will set the ETag as the MD5 of the response contents.
 * 
 * @author icoloma
 */
@SSLPolicy(SSLPolicyType.DO_NOT_MODIFY)
public class ResourcesAction extends AbstractAction {

	/** 
	 * the name of the resource GET parameter. 
	 * This parameter is informational for the user (to see the script name in the browser),
	 * and will never be used by the server for security reasons 
	 */
	public static final String RESOURCE = "resourceName";
	
	/** the resource bundle name */
	public static final String BUNDLE = "bundle";
	
	/** the index of the resource inside the bundle. If "all", the entire bundle will be returned */
	public static final String INDEX = "resourceIndex";
	
	/** the value top retrieve an entire resource bundle */
	public static final String INDEX_ALL = "all";
	
	/** the name of the md5 GET parameter */
	public static final String MD5 = "md5";
	
	/** if present, use this MD5 to check if the browser data is up-to-date */
	private String md5;

	/** return this resource inside the requested resourceBundle (integer value). If "all", return the entire WebResourceBundle */
	private String resourceIndex;
	
	/** the javascript/css/image file to return */
	private String resourceName;
	
	/** used when invoking getResource() */
	private String bundle;
	
	/** if resource != null, this is the resolved resource contents */
	private WebResourceData resolvedResource;
	
	private static Log log = Log.getLog();
	
	/**
	 * If a MD5 attribute has been passed and it's the same of the actual MessageRepository contents,
	 * set the response to cache forever.
	 * If no MD5 GET parameter has been passed, use it as ETag as usual.
	 */
	@Cache(on="getJsMessagesRepository")
	public CacheControl cacheMessagesRepository() {
		CacheControl c = new CacheControl();
		MessagesRepository repository = getContext().getRequest().getMessagesRepository();
		c.setEtag(repository.getMD5());
		if (md5 != null && md5.equals(c.getEtag())) {
			c.setCacheForever();
		}
		return c;
	}
	
	/**
	 * Converts the {@link MessagesRepository} instance to a javascript JSON presentation.
	 * If the provided MD5 parameter is the same of the current one, the response will be set 
	 * to cache forever.
	 * @return a javascript version of the {@link MessagesRepository} for this locale
	 */
	@Event(defaultEvent=true)
	@Path("/i18n/{md5}")
	public Resolution getJsMessagesRepository() {
		MessagesRepository messagesRepository = getMessagesRepository();
		AbstractHttpResolution r;
		if (supportsGzip()) {
			r = new InputStreamResolution(new ByteArrayInputStream(messagesRepository.getGzippedJson()));
			r.setHeader(ResponseHeaderNames.CONTENT_ENCODING, "gzip");
		} else {
			r = new StringResolution(messagesRepository.toJSON());
		}
		r.setContentType(ContentTypeNames.JSON);
		r.setCharset("UTF-8");
		return r;
	}
	
	@Cache(on="getResource")
	public CacheControl cacheResource() throws IOException {
		CacheControl c = new CacheControl();
		try {
			WebResourcesRepository repository = Config.getInstance().getWebResourcesRepository();
			resolvedResource = INDEX_ALL.equals(resourceIndex)? repository.getResourceData(bundle) : repository.getResourceData(bundle, Integer.parseInt(resourceIndex), resourceName);
			c.setEtag(resolvedResource.getMd5());
			if (md5 != null && md5.equals(c.getEtag())) {
				c.setCacheForever();
			}
		} catch (NumberFormatException e) {
			// do nothing here; it will return 404, since the resolvedResource is null
			log.error(e, "Bad resourceIndex number: " + resourceIndex);
		} catch (FileNotFoundException e) {
			// do nothing here; it will return 404, since the resolvedResource is null
			log.error(e, e);
		}
		return c;
	}
	
	/**
	 * If Config.development = false, retrieves an entire concatenated bundle
	 * and returns it gzipped (if the browser supports gzip).
	 * @return the requested resource, gzipped if supported by the client
	 */
	@Path("/{bundle}/{resourceIndex}/{md5}/{resourceName?}")
	public Resolution getResource() throws FileNotFoundException {
		if (resolvedResource == null) {
			// maybe it's a file that exists in the given location
			InputStream resource = getContext().getServletContext().getResourceAsStream(getContext().getRequest().getServletPath());
			return resource == null? new HttpErrorResolution(404) : new InputStreamResolution(resource);
		}
		
		InputStreamResolution resolution;
		if (supportsGzip()) { // accepts gzip
			resolution = new InputStreamResolution(null, resolvedResource.getGzippedInputStream(), resolvedResource.getGzippedLength());
			resolution.setHeader(ResponseHeaderNames.CONTENT_ENCODING, "gzip");
		} else {
			resolution = new InputStreamResolution(null, resolvedResource.getInputStream(), resolvedResource.getLength());
		}
		resolution.setContentType(resolvedResource.getType().getContentType());
		resolution.setCharset(resolvedResource.getCharset());
		return resolution;
	}
	
	private boolean supportsGzip() {
		String header = getContext().getRequest().getHeader(RequestHeaderNames.ACCEPT_ENCODING);
		return header != null && header.contains("gzip");
	}
	
}
