/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation.processor;

import javax.persistence.Basic;
import javax.persistence.Column;

import org.loom.annotation.validation.NestedAnnotations;
import org.loom.mapping.Event;
import org.loom.validator.AbstractPropertyValidator;
import org.loom.validator.AbstractRequiredValidator;
import org.loom.validator.NumberValidator;
import org.loom.validator.RequiredValidatorFactory;
import org.loom.validator.StringValidator;


/**
 * Processes JPA annotations.
 * This processor will look inside any class that has been annotated with {@link NestedAnnotations}
 * and add validators corresponding to the metadata that is used to generate the database.
 * {@link Basic} and {@link Column} are supported.
 * @author icoloma
 *
 */
public class ColumnAnnotationProcessor extends AbstractPropertyAnnotationProcessor<Column, Column> {

	public ColumnAnnotationProcessor() {
		super(Column.class, null);
	}

	@Override
	protected void process(Event event, Column column, String propertyPath) {
		
		// required validator
		Class<?> propertyClass = event.getPropertyClass(propertyPath);
		if (!column.nullable()) {
			AbstractRequiredValidator<?> validator = RequiredValidatorFactory.createValidator(propertyClass);
			validator.setPropertyPath(propertyPath);
			event.addValidator(validator);
		}
		
		// string validator
		AbstractPropertyValidator validator = null;
		if (String.class.equals(propertyClass)){
			StringValidator sv = new StringValidator();
			sv.setMaxLength(column.length());
			validator = sv;
		} else if (Number.class.isAssignableFrom(propertyClass) && (column.precision() != 0 || column.scale() != 0)) {
			
		// number validator
			NumberValidator nv = new NumberValidator();
			if (column.scale() != 0) {
				nv.setScale(column.scale());
			}
			if (column.precision() != 0) {
				nv.setPrecision(column.precision());
			}
			validator = nv;
		}
		
		if (validator != null) {
			validator.setPropertyPath(propertyPath);
			event.addValidator(validator);
		}
			
	}

}
