/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;


import org.loom.exception.MalformedPropertyPathException;
import org.loom.validator.ValidationRequest;

/**
 * A nesting level in a property path
 * @author icoloma
 */
public abstract class Node {

	/** the next node in the chain */
	private Node child;

	/** 
	 * for single values, this field does the real property get/set of values
	 * for multiple values, this field is the collection/array/map attribute 
	 */
	private PropertyWrapper propertyWrapper;
	
	/**
	 * Sets a value to the property at the end of this chain
	 * @param container The current property node (starts at the root of the chain)
	 * @param value the value to assign
	 * @param index the current index inside the property chain 
	 */
	public abstract void setValue(Object container, Object value, String[] propertyPath, int index) throws MalformedPropertyPathException;
	
	/**
	 * Gets the value of the property at the end of this chain
	 * @param container The current property node (starts at the root of the chain)
	 * @param propertyPath the full path to the requested property
	 * @param index the current index inside the property chain 
	 * @return the value at the end of this chain, may be null
	 */
	public abstract Object getValue(Object container, String[] propertyPath, int index) throws MalformedPropertyPathException;

	/**
	 * @return the container class for any child nodes. For simple nodes this would be the same as the property class, but
	 * collections will return the item class and Maps the value  class.
	 */
	public final Class<?> getNextNodeClass() {
		return propertyWrapper.getNextPropertyClass();
	}
	
	
	public Node getChild() {
		return child;
	}

	public void setChild(Node child) {
		this.child = child;
	}

	public String getPropertyName() {
		return propertyWrapper.getPropertyName();
	}
	
	/**
	 * @param wrapper the propertyWrapper of this field. 
	 */
	public void setPropertyWrapper(PropertyWrapper wrapper) {
		this.propertyWrapper = wrapper;
	}

	/**
	 * @return true if this node is a leaf node, the last node in the property chain
	 */
	public boolean isLeaf() {
		return child == null;
	}
	
	/**
	 * Gets the index part of a node. "foo[index]" returns the index as String.
	 * Can be used by Maps, arrays and Lists.
	 * @return the index of this node, null if none
	 */
	protected String getIndex(String nodeName) {
		int init = nodeName.indexOf('[');
		int end = -1;
		if (init != -1)
			end = nodeName.indexOf(']');
		if (init == -1 || end == -1 || init + 1 == end)
			return null;
		return nodeName.substring(init + 1, end);
	}
	
	/**
	 * Gets the index as Integer
	 * If there is no index, return -1
	 */
	protected int getIndexAsInt(String nodeName) {
		try {
			String indexAsString = getIndex(nodeName);
			return indexAsString == null? -1 : Integer.parseInt(indexAsString);
		} catch (NumberFormatException e) {
			throw new MalformedPropertyPathException("Expected integer index at " + nodeName);
		}
	}
	
	/**
	 * Applies validation to this node. Handles the null container case (where an intermediate node is null) and
	 * propagates the call to applyValidationImpl().
	 */
	public void applyValidation(Object container, ValidationRequest request) {
		if (container == null) {
			request.push(propertyWrapper.getPropertyName());
			request.validate(null);
			request.pop();
		} else {
			applyValidationImpl(container, request);
		}
	}

	/**
	 * Applies a validation. If there is more than one value involved in this validation (i.e. lists,
	 * maps, or arrays), validates every node.
	 * @param container the container where to apply this validation. It cannot be null.
	 * @param request the validation to be performed 
	 */
	protected abstract void applyValidationImpl(Object container, ValidationRequest request);

	public PropertyWrapper getPropertyWrapper() {
		return propertyWrapper;
	}
	
}
