package org.loom.exception;

import java.io.IOException;

import javax.servlet.ServletException;

import org.loom.i18n.Messages;
import org.loom.log.Log;
import org.loom.resolution.HttpErrorResolution;
import org.loom.resolution.Resolution;
import org.loom.servlet.HttpMethod;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.names.RequestHeaderNames;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.url.UrlBuilder;
import org.loom.util.ExceptionUtils;

/**
 * Forwards all returned exceptions to the error 500 page
 *  
 * @author icoloma
 *
 */
public class DefaultExceptionHandler implements ExceptionHandler {

	private static Log log = Log.getLog();
	
	/** true to throw any unknown exception instead of forwarding a HTTP 500 */
	private boolean rethrowExceptions = false;
	
	public Throwable translate(Throwable applicationException) {
		return ExceptionUtils.getNestedException(applicationException);
	}
	
	public void handle(LoomServletRequest request, LoomServletResponse response, Throwable exception) throws IOException, ServletException {
		exception = translate(exception);
		log.error(exception, exception);
		
		if (exception instanceof LocaleAwareThrowable && request.getHeader(RequestHeaderNames.REFERER) != null && request.getHttpMethod() != HttpMethod.GET) {
			// if the exception is locale aware and we know the source page, treat it as a validation error
			request.getParsedAction().getMessages().addMessage(((LocaleAwareThrowable)exception).getLocaleAwareMessage());
			redirectWithErrors(request, response);
			return;
		} 
		
		Resolution resolution = null;
		if (exception instanceof HttpThrowable) {
			resolution = new HttpErrorResolution((HttpThrowable)exception);
		} 
		
		if (resolution == null) {
			if (rethrowExceptions) {
				throw exception instanceof ServletException? (ServletException) exception : new ServletException(exception);
			}
			resolution = new HttpErrorResolution(500, exception);
		}
		resolution.resolve(request, response);
	}
	
	public void redirectWithErrors(LoomServletRequest request, LoomServletResponse response) throws IOException, ServletException {
		Messages messages = request.getParsedAction().getMessages();
		String referrer = request.getHeader(RequestHeaderNames.REFERER);
		if (referrer == null || request.isAjax()) { // no source form to forward to, throw 400
			throw new UnhandledValidationErrorsException(messages, request.getMessagesRepository());
		}
		
		request.getParsedAction().getMessages().saveForRedirect(request);
		request.getRequestParameters().saveForRedirect(request);
		UrlBuilder builder = request.createUrl(referrer);
		builder.replace(RequestParameterNames.REDIRECT_ERROR, "true");
		response.sendRedirect(builder.getURL());
	}

	public void setRethrowExceptions(boolean rethrowExceptions) {
		this.rethrowExceptions = rethrowExceptions;
	}
}
