/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.exception;

import java.util.Locale;
import java.util.Map;

import org.loom.config.Config;
import org.loom.i18n.Message;
import org.loom.i18n.MessagesRepository;

/**
 * Locale-dependent exceptions. This may be handled as validation errors if there is a source form. 
 * @author icoloma
 */
public class LocaleAwareException extends RuntimeException implements LocaleAwareThrowable {

	/** the message to display */
	private Message message;
	
	/** the message to display when logging */
	private String legacyMsg;
	
	/**
	 * @param message the message to be printed in logs and where the current locale cannot be resolved
	 * @param messageKey the key of the message to be looked for in the current {@link MessagesRepository} instance
	 * @param cause the cause of this exception
	 * @deprecated use {@link LocaleAwareException#LocaleAwareException(String, Throwable)} instead
	 */
	public LocaleAwareException(String message, String messageKey, Throwable cause) {
		this(messageKey, cause);
		this.legacyMsg = message;
	}
	
	/**
	 * @param message the message to be printed in logs and where the current locale cannot be resolved
	 * @param messageKey the key of the message to be looked for in the current {@link MessagesRepository} instance
	 * @deprecated use {@link LocaleAwareException#LocaleAwareException(String)} instead
	 */
	public LocaleAwareException(String message, String messageKey) {
		this(messageKey, (Throwable) null);
		this.legacyMsg = message;
	}
	
	/**
	 * Note that this constructor will display a meaningless message (the message key and params) if there is no 
	 * @param messageKey the key of the message to be looked for in the current {@link MessagesRepository} instance
	 */
	public LocaleAwareException(String messageKey) {
		this(messageKey, (Throwable) null);
	}

	/**
	 * Note that this constructor will display a meaningless message (the message key and params) if there is no 
	 * @param messageKey the key of the message to be looked for in the current {@link MessagesRepository} instance
	 * @param cause the cause of this exception
	 */
	public LocaleAwareException(String messageKey, Throwable cause) {
		super(null, cause);
		this.message = new Message(messageKey);
		//this.message.addArg("exception", this);
	}
	
	/**
	 * This constructor receives the i18n message to display to the user.
	 * Note that this constructor will display a meaningless message (the message key and params) if there is no 
	 * {@link MessagesRepository} instance available.
	 * @param message the message to be printed in logs and where the current locale cannot be resolved
	 */
	public LocaleAwareException(Message message) {
		super();
		this.message = message;
	}
	
	public Message getLocaleAwareMessage() {
		return message;
	}

	public LocaleAwareException addArg(String arg, Object value) {
		message.addArg(arg, value);
		return this;
	}

	public LocaleAwareException addTranslatedArg(String arg, Object value) {
		message.addTranslatedArg(arg, value);
		return this;
	}

	public Map<String, Object> getArgs() {
		return message.getArgs();
	}

	public String getMessageKey() {
		return message.getMessageKey();
	}

	public Map<String, Object> getTranslatedArgs() {
		return message.getTranslatedArgs();
	}

	@Override
	public String getMessage() {
		if (legacyMsg != null) {
			return legacyMsg;
		}
		Config config = Config.getInstance();
		if (config == null) {
			return message.getMessageKey();
		}
		MessagesRepository repository = config.getMessagesRepositoryFactory().getRepository(Locale.getDefault());
		return repository.translateMessage(message);
	}
	

}
