/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.filter;

import java.io.IOException;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.loom.log.Log;
import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.ResponseListener;
import org.loom.util.UrlMatcher;

/**
 * Expands oscache to add filtered match. 
 * 
 * @author icoloma
 */
public class StaticResourcesFilter extends AbstractLoomFilter implements ResponseListener {

	/** the list of configured patterns */
	private UrlMatcher matcher = new UrlMatcher();

	/** the list of methods that are not cached */
	private Set<String> skipMethods;
	
	/** the cache data */
	private CacheControl cacheControl;
	
	private static Log log = Log.getLog();
	
	/**
	 * Expected init argument: 
	 * patterns: list of regex for  matching patterns, separated by commas, spaces, tabs or newlines
	 */
	@Override
	public void init(FilterConfig filterConfig) {
		
		cacheControl = new CacheControl();
		cacheControl.setCacheForever();
		
		String patterns = filterConfig.getInitParameter("includes");
		if (patterns !=  null) {
			this.matcher.setInclude(patterns != null? StringUtils.split(patterns, ",\n\t ") : new String[] { "-[\\d.]+.*/", "-[\\d.]+\\.\\w+$" });
		} 
		patterns = filterConfig.getInitParameter("excludes");
		if (patterns !=  null) {
			this.matcher.setExclude(StringUtils.split(patterns, ",\n\t "));
		} 
		
		skipMethods = new TreeSet<String>();
		skipMethods.add("PUT");
		skipMethods.add("POST");
		skipMethods.add("DELETE");
	}

	@Override
	public void doFilter(LoomServletRequest request, LoomServletResponse response, FilterChain chain)
			throws IOException, ServletException {
		if (matcher.matches(request) && !skipMethods.contains(request.getMethod())) {
			log.debug("Request '", request.getServletPath(), "' is being marked as CACHE FOREVER");
			response.addListener(this);
		}
		chain.doFilter(request, response);
	}

	@Override
	public void destroy() {
	}

	/**
	 * If the response is not a 404, 403, etc, sets the cache headers accordingly
	 */
	public void responseStart(LoomServletResponse response) {
		if (response.getStatus() == HttpServletResponse.SC_OK) {
			cacheControl.writeHeaders(response);
		}
	}
	
}
