/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.i18n;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormatSymbols;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;
import java.util.zip.GZIPOutputStream;

import org.apache.commons.io.IOUtils;
import org.loom.config.ResourcesModificationListener;
import org.loom.json.JsonMarshaller;
import org.loom.log.Log;
import org.loom.util.PropertyUtils;
import org.loom.util.StringUtils;
import org.mvel2.CompileException;
import org.springframework.core.io.Resource;


/**
 * Repository for messages that may be translated.
 * This repository is specific for a given Locale.
 * @author Ignacio Coloma
 */
public class MessagesRepository implements ResourcesModificationListener {
	
	/** Some important keys */
	public static final String DECIMAL_FORMAT_KEY = "loom.format.decimal";
	public static final String DECIMAL_SEPARATOR_KEY = "loom.format.decimalSeparator";
	public static final String GROUPING_SEPARATOR_KEY = "loom.format.groupingSeparator";
	public static final String JSNUMBER_FORMAT_KEY = "loom.format.number";

	/** the strategy to follow when looking for a message that is missing fom the ResouceBundle */
	private MissingMessageHandler missingMessagesHandler;

	/** the list of Resource that are read by this MessagesRepository */
	private List<Resource> resources;
	
	/** the underlying messages */
	private Map<String, String> contents = new HashMap<String, String>(256, .75f);
	
	/** the MVEL compiled messages for this locale */
	private Map<String, CompiledMessage> compiledMessages = new ConcurrentHashMap<String, CompiledMessage>(100);
	
	/** the Locale for this repository */
	private Locale locale;
	
	/** this instance as a JSON object to be used by javascript code */
	private String json;
	
	/** the JSON contents, gzipped */
	private byte[] gzippedJson;
	
	/** the MD5 hash of the JSON representation of this object */
	private String md5;
	
	/** message keys that should be send to the browser. */
	private Set<String> browserMessages = new TreeSet<String>();
	
	/** the timestamp where this object was read for the last time */
	private Date lastModified = new Date();
	
	private static Log log = Log.getLog();
	
	public MessagesRepository(Locale locale) {
		this.locale = locale;
	}

	/**
	 * Returns a key translated to the current locale.
	 * This method is similar in behaviour to ResourceBundle.getString(),
	 * but will return a [missing:key] if the key could not be found.
	 */
	public String getString(String key) {
		if (key == null || key.length() == 0) {
			return null;
		}
		String result = contents.get(key);
		return result == null? missingMessagesHandler.handleMissingKey(key) : result;
	}
	
	/**
	 * Returns a key translated to the current locale.
	 * This method is similar in behaviour to ResourceBundle.getString(),
	 * but will return null if the resource is not found
	 */
	public String getNullableString(String key) {
		return contents.get(key);
	}
	
	/**
	 * Returns a key translated to the current locale.
	 * This method throws an Exception if the key is missing
	 */
	public String getRequiredString(String key) {
		String result = contents.get(key);
		if (result == null) {
			throw new IllegalArgumentException("Could not find required string: " + key);
		}
		return result;
	}
	
	/**
	 * Returns a key translated to the current locale.
	 * If the key is a nested path like a.b.c, first tries a.b.c, then b.c, then c.
	 * If none of them returns a valid translation, returns the same as getString() 
	 */
	public String guessString(String paramName) {
		for (String s = paramName; s != null; s = PropertyUtils.removeRootNode(s)) {
			String result = getNullableString(s);
			if (result != null) {
				if (s != paramName) { // remember, to skip guessing the next time
					contents.put(paramName, result);
				}
				return result;
			}
		}
		return missingMessagesHandler.handleMissingParameter(paramName);
	}
	
	/**
	 * Returns the translated Message for the current Locale.
	 * Merges a Velocity template with the given args.
	 * @param message The message to be translated
	 * @return The translated message
	 */
	public String translateMessage(Message message) {
		if (message.isDeserialized()) {
			return message.getSerializedValue();
		}
			
		String key = message.getMessageKey();
		CompiledMessage compiledMessage = compiledMessages.get(key);
		if (compiledMessage == null) {
			compiledMessage = MessageCompiler.compile(getRequiredString(key));
			compiledMessages.put(key, compiledMessage);
		}
		
		// add arguments, 'as is' and i18n
		Map<String, Object> args = new HashMap<String, Object>(message.getArgs());
		Map<String, Object> i18nArgs = message.getTranslatedArgs();
		for (Map.Entry<String, Object> entry : i18nArgs.entrySet()) { 
			String k = entry.getKey();
			Object value = entry.getValue();
			args.put(k, value == null? "" : guessString(value.toString()));
		}

		// eval translated message
		try {
			return compiledMessage.eval(args);
		} catch (CompileException e) {
			throw new RuntimeException("Cannot evaluate message '" + getRequiredString(message.getMessageKey()) + "': " + e.getMessage(), e);
		}
	}
	
	/**
	 * Initializes this MessagesRepository instance with the contents of the provided files
	 */
	public void initialize() {
		resourcesModified(resources);
	}

	public void resourcesModified(List<Resource> res) {
		for (Resource resource : res) {
			log.debug("Loading ", resource);
			InputStream input = null;
			try {
				input = resource.getInputStream();
				Properties p = new Properties();
				p.load(input);
				addProperties(p);
			} catch (IOException e) {
				throw new RuntimeException(e);
			} finally {
				IOUtils.closeQuietly(input);
			}			
		}
	}


	/**
	 * Returns the list of messages
	 * @return The translated messages
	 */
	public String translateMessages(List<Message> messages) {
		StringBuilder buffer = new StringBuilder().append("<ul>");
		for (Message message : messages) {
			buffer.append("<li>").append(translateMessage(message)).append("</li>\n");
		}
		buffer.append("</ul>\n");
		return buffer.toString();
	}

	/**
	 * Get the DecimalFormatSymbols associated to the current Locale.
	 * This method returns the DecimalFormatSymbols associated to the locale by the JVM,
	 * modified by the user if there is one decimalSeparator or groupingSeparator
	 * in the properties file.
	 */
	public DecimalFormatSymbols createDecimalFormatSymbols() {
		DecimalFormatSymbols symbols = new DecimalFormatSymbols(getLocale());
		try {
			String decimalSeparator = getRequiredString(DECIMAL_SEPARATOR_KEY);
			symbols.setDecimalSeparator(decimalSeparator.charAt(0));
		} catch (MissingResourceException e) {
			// no decimal separator defined, so the default for the locale is ok
		}
		try {
			String groupingSeparator = getRequiredString(GROUPING_SEPARATOR_KEY);
			symbols.setGroupingSeparator(groupingSeparator.charAt(0));
		} catch (MissingResourceException e) {
			// no grouping separator defined, the default for the locale is ok
		}
		return symbols;
	}
	
	/**
	 * @return a JSON representation of the error messages and other i18n 
	 * contents in this repository, to be used by the javascript validation library
	 */
	public String toJSON() {
		if (json == null) {
			synchronized(this) { // guarantee that two threads do not initialize this at once
				if (json == null) {
					JsonMarshaller marshaller = new JsonMarshaller("loom.messages");
					marshaller.setDeclare(false);
					marshaller.add("lang", locale.getLanguage());
					
					// add the number format
					DecimalFormatSymbols symbols = createDecimalFormatSymbols();
					String numberFormat = "^-?(((\\d{1,3})(\\" + symbols.getGroupingSeparator() + "\\d{3})*)|(\\d+))(\\" + symbols.getDecimalSeparator() + "\\d+)?$";
					marshaller.add(JSNUMBER_FORMAT_KEY, Pattern.compile(numberFormat));
					
					// add JSON keys
					for (String key: browserMessages) {
						marshaller.add(key, guessString(key));
					}
					
					// make possible declaring this object before including core.js
					json = "var loom = window['loom'] || {};\n" + marshaller.toString();
					
					// @deprecated : loom.i18n.resources will be removed by loom 1.1
					// (by removing this line)
					// json += "loom.i18n = { resources: loom.messages }\n";
					
					// gzip
					ByteArrayInputStream bis = null;
					ByteArrayOutputStream bos = null;
					GZIPOutputStream gos = null;
					try {
						bis = new ByteArrayInputStream(json.getBytes("UTF-8"));
						bos = new ByteArrayOutputStream();
						gos = new GZIPOutputStream(bos);
						IOUtils.copy(bis, gos);
					} catch (IOException e) {
						// this should not happen
						throw new RuntimeException(e);
					} finally {
						IOUtils.closeQuietly(bis);
						IOUtils.closeQuietly(bos);
						IOUtils.closeQuietly(gos);
					}
					gzippedJson = bos.toByteArray();
					
					// calculate the MD5 checksum
					md5 = StringUtils.toMD5(json);
				}
			}
		}
		return json;
	}
	
	/**
	 * @return the MD5 hash of the string representation of the current contents
	 */
	public String getMD5() {
		// lazily initialize the JSON representation
		toJSON();
		return md5;
	}

	public Locale getLocale() {
		return locale;
	}

	public MissingMessageHandler getMissingMessagesHandler() {
		return missingMessagesHandler;
	}

	public void setMissingMessagesHandler(MissingMessageHandler missingMessagesHandler) {
		this.missingMessagesHandler = missingMessagesHandler;
	}

	/**
	 * Add all entries from a Properties file into this instance contents
	 * @param properties
	 */
	public void addProperties(Properties properties) {
		for (Map.Entry<Object, Object> entry : properties.entrySet()) {
			addString((String)entry.getKey(), (String)entry.getValue());
		}
	}
	
	/**
	 * Add one entry to this MessagesRepository instance
	 */
	public void addString(String name, String value) {
		// memory and lookup speed optimization, see http://satukubik.com/2009/01/06/java-tips-memory-optimization-for-string/
		name = name.intern();
		contents.put(name, value);
		
		// validation and conversion errors will be sent to the browser
		if (name.startsWith("loom.")) {
			addBrowserMessage(name);
		}
	}
	
	/**
	 * @return the size of its contents. This size may change with time, as guessed keys are added
	 * lazily.
	 */
	public int size() {
		return contents.size();
	}
	
	/**
	 * @return an unmodifiable {@link Map} adapter to this instance. This Map will see changes made to this 
	 * {@link MessagesRepository} class in real time, but cannot modify it.  
	 */
	public Map<String, String> asMap() {
		return new MessagesRepositoryToMapAdapter(this);
	}

	public Set<String> getBrowserMessages() {
		return browserMessages;
	}

	public void addBrowserMessage(String browserMessage) {
		if (this.browserMessages.add(browserMessage) && json != null) {
			this.json = null;
			this.md5 = null;
		}
	}
	
	public void addBrowserMessages(Set<String> browserMessages) {
		for (String key : browserMessages) {
			addBrowserMessage(key);
		}
	}

	public Date getLastModified() {
		return lastModified;
	}

	public void setLastModified(Date lastModified) {
		this.lastModified = lastModified;
	}

	public byte[] getGzippedJson() {
		return gzippedJson;
	}

	public List<Resource> getResources() {
		return resources;
	}

	public void setResources(List<Resource> resources) {
		this.resources = resources;
	}
	
	Map<String, String> getContents() {
		return contents;
	}

}
