/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.interceptor;

import static org.loom.url.UrlBuilder.INSECURE_SCHEME;
import static org.loom.url.UrlBuilder.SECURE_SCHEME;

import org.loom.annotation.SSLPolicy;
import org.loom.annotation.SSLPolicyType;
import org.loom.log.Log;
import org.loom.mapping.Event;
import org.loom.mapping.ParsedAction;
import org.loom.resolution.RedirectResolution;
import org.loom.resolution.RedirectResolutionImpl;
import org.loom.resolution.Resolution;
import org.loom.servlet.HttpMethod;
import org.loom.servlet.LoomServletRequest;
import org.loom.tags.Form;
import org.loom.tags.HtmlTag;
import org.loom.tags.Link;
import org.loom.url.UrlBuilder;
import org.loom.validator.FormDecorator;
import org.loom.validator.LinkDecorator;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.security.web.PortMapper;
import org.springframework.security.web.PortMapperImpl;
import org.springframework.security.web.PortResolver;
import org.springframework.security.web.PortResolverImpl;
import org.springframework.stereotype.Component;

/**
 * Checks that the incoming request has the expected security requirements
 * @author icoloma
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class SSLPolicyInterceptor implements LinkDecorator, FormDecorator, PopulateInterceptor, InitializingBean {
	
	/** will provide the http-to-https port mapping */
	@Autowired(required=false)
	private PortMapper portMapper;
	
	@Autowired(required=false)
	private PortResolver portResolver;
	
	/** the expected SSLPolicy */
	private SSLPolicyType policy;
	
	private static Log log = Log.getLog();
	
	public void afterPropertiesSet() {
		if (portMapper == null) {
			portMapper = new PortMapperImpl();
		}
		if (portResolver == null) {
			portResolver = new PortResolverImpl();
			((PortResolverImpl)portResolver).setPortMapper(portMapper);
		}
	}
	
	public Resolution beforePopulate(ParsedAction action) {
		LoomServletRequest request = action.getRequest();
		boolean secure = request.isSecure();
		Integer port = portResolver.getServerPort(request);
		Integer redirectPort = null;
		String redirectScheme = null;
		
		if (secure && policy == SSLPolicyType.REQUIRES_INSECURE) {
			redirectScheme = INSECURE_SCHEME;
			redirectPort = portMapper.lookupHttpPort(port);
		} else if (!secure && policy == SSLPolicyType.REQUIRES_SECURE) {
			redirectScheme = SECURE_SCHEME;
			redirectPort = portMapper.lookupHttpsPort(port);
		}
		
		if (redirectScheme != null) {
			if (request.getHttpMethod() != HttpMethod.GET) {
				throw new RuntimeException("Request does not have the correct protocol (expected: " + redirectScheme + " on port " + redirectPort + ". Since this is a " + request.getHttpMethod() + " request, we cannot redirect and just panic");
			}
			String serverName = getServerName(request);
			log.debug("Redirecting request to ", redirectScheme, " with port ", redirectPort);
			
			Event event = action.getEvent();
			RedirectResolution resolution = new RedirectResolutionImpl(event.getActionMapping().getActionClass(), event.getName());
			resolution.setScheme(redirectScheme);
			resolution.setPort(redirectPort);
			if (serverName != null) {
				resolution.setHostname(serverName);
			}
			resolution.setIncludeRequestParameters(true);
			return resolution;
		}
		
		return null;
	}

	/**
	 * If the current request does not have the expected protocol (http/https),
	 * modify the links pointing to this event.
	 */
	public void decorate(Link link) {
		decorate(link.getUrl(), link);
	}
	
	/**
	 * If the current request does not have the expected protocol (http/https),
	 * modify the forms pointing to this event.
	 */
	public void decorate(Form form) {
		decorate(form.getUrl(), form);
	}

	protected void decorate(UrlBuilder url, HtmlTag tag) {
		LoomServletRequest request = tag.getRequest();
		boolean secure = request.isSecure();
		Integer redirectPort = null;
		if (secure && policy == SSLPolicyType.REQUIRES_INSECURE) {
			url.setScheme(INSECURE_SCHEME);
			redirectPort = portMapper.lookupHttpPort(request.getServerPort());
		} else if (!secure && policy == SSLPolicyType.REQUIRES_SECURE) {
			url.setScheme(SECURE_SCHEME);
			redirectPort = portMapper.lookupHttpsPort(request.getServerPort());
		}
		if (redirectPort != null) {
			url.setPort(redirectPort);
		}
		url.setHostname(getServerName(request));
	}
	
	/**
	 * Calculate the server name to redirect to. This method should be overriden if the 
	 * target server differs between SSL and not-SSL requests.
	 * This method is only invoked if the incoming request does not match the {@link SSLPolicy}
	 * specified for this interceptor, and should return null to use the request-provided 
	 * server name.
	 */
	protected String getServerName(LoomServletRequest request) {
		return null;
	}

	@Override
	public String toString() {
		return "SSLPolicyInterceptor { " + policy + " }"; 
	}
	
	public PortMapper getPortMapper() {
		return portMapper;
	}

	public void setPortMapper(PortMapper portMapper) {
		this.portMapper = portMapper;
	}

	public SSLPolicyType getPolicy() {
		return policy;
	}

	public void setPolicy(SSLPolicyType policy) {
		this.policy = policy;
	}

}
