package org.loom.interceptor;

import java.util.Set;
import java.util.UUID;
import java.util.concurrent.CopyOnWriteArraySet;

import javax.servlet.http.HttpSession;

import org.loom.exception.LocaleAwareException;
import org.loom.log.Log;
import org.loom.mapping.ParsedAction;
import org.loom.resolution.Resolution;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.names.CssClassNames;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.servlet.names.SessionAttributeNames;
import org.loom.tags.Form;
import org.loom.tags.Link;
import org.loom.validator.FormDecorator;
import org.loom.validator.LinkDecorator;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;

/**
 * Guarantees that an event is invoked at most once.
 * This class will assign new tokens to links and forms pointing to this event,
 * storing them in the user session. If an incoming request does not have a valid token, 
 * it will be rejected.
 * 
 * @author icoloma
 */
@Scope(BeanDefinition.SCOPE_SINGLETON)
public class SingleUseTokenInterceptor implements ValidateInterceptor, LinkDecorator, FormDecorator {

	private static Log log = Log.getLog();

	public Resolution beforeValidate(ParsedAction action) {
		LoomServletRequest request = action.getRequest();
		String token = request.getParameter(RequestParameterNames.SINGLE_USE_TOKEN);
		if (!validateToken(request, token)) {
			rejectRequest(request);
		}
		return null;
	}
	
	protected void rejectRequest(LoomServletRequest request) {
		String token = request.getParameter(RequestParameterNames.SINGLE_USE_TOKEN);
		throw new LocaleAwareException("Cannot process request (wrong token value: " + token + ")",
				"loom.validation.singleUseTokenFailed").addArg("token", token);
	}

	/**
	 * Create a new token value and store it in the user session
	 * @param request
	 * @return the token value
	 */
	protected String createNewToken(LoomServletRequest request) {
		HttpSession session = request.getSession();
		Set<String> tokens = getTokensContainer(session);
		if (tokens == null) {
			synchronized(session) {
				tokens = getTokensContainer(session);
				if (tokens == null) {
					tokens = new CopyOnWriteArraySet<String>();
					session.setAttribute(SessionAttributeNames.SINGLE_USE_TOKENS_CONTAINER, tokens);
				}
			}
		}
		String newToken = UUID.randomUUID().toString();
		tokens.add(newToken);
		log.debug("Creating new token ", newToken, " for session ", session.getId());
		return newToken;
	}

	@SuppressWarnings("unchecked")
	private Set<String> getTokensContainer(HttpSession session) {
		return (Set<String>) session.getAttribute(SessionAttributeNames.SINGLE_USE_TOKENS_CONTAINER);
	}
	
	/**
	 * Validate a token and removes it from the session
	 * @param token the token value to validate
	 * @return true if the token is valid, false otherwise
	 */
	public boolean validateToken(LoomServletRequest request, String token) {
		if (token == null) {
			return false;
		}
		HttpSession session = request.getSession();
		log.debug("Validating token ", token, " for session ", session.getId());
		Set<String> tokens = getTokensContainer(session);
		return tokens.remove(token);
	}
	
	public void decorate(Link link) {
		link.addCssClass(CssClassNames.SINGLE_USE_TOKEN);
		link.getUrl().add(RequestParameterNames.SINGLE_USE_TOKEN, createNewToken(link.getRequest()));
	}

	public void decorate(Form form) {
		form.addCssClass(CssClassNames.SINGLE_USE_TOKEN);
		form.addHiddenField(RequestParameterNames.SINGLE_USE_TOKEN, createNewToken(form.getRequest()));
	}
	
}
