/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.mapping;

import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.loom.action.Action;
import org.loom.binding.PropertyBinder;
import org.loom.binding.PropertyBinderMap;
import org.loom.converter.Converter;
import org.loom.exception.EventNotFoundException;
import org.loom.exception.PropertyNotFoundException;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.util.PropertyUtils;
import org.loom.validator.PathMatcher;
import org.loom.validator.PropertyValidator;

/**
 * The mapping information about one Action class. 
 * @author icoloma
 */
public class ActionMapping {

	/** the action class */
	private Class<? extends Action> actionClass;
	
	/** the spring bean name */
	private String beanName;
	
	/** the default event, if any. Default null */
	private Event defaultEvent;
	
	/** the list of available events: the key is the event name, the value is the method to be called */
	private Map<String, Event> events;
	
	/** the list of PropertyBinders */
	private PropertyBinderMap propertyBinderMap;

	/** the short name of this action */
	private String shortName;
	
	/** 
	 * the url handle. These are the first substring that relate a request with its mapping.
	 * It must start with '/' and not end with '/' 
	 */
	private String handle;
	
	/**
	 * the default forward folder. Defaults to the default action mapping handle
	 */
	private String forwardFolder;
	
	/** 
	 * the set of classes that affect this mapping. When class reloading is active, 
	 * any changes to one of these will trigger the reload of this ActionMapping instance.
	 * This keeps track of classes with annotations used to configure validations.
	 */ 
	private Set<Class<?>> reloadingClasses;
	
	private Log log = Log.getLog();
	
	/**
	 * @return event metadata for the required event, the default event if none. 
	 * @throws EventNotFoundException if the event is not found, or it has not been specified and no default event exists for this action.
	 */
	public Event getEvent(String event) {
		if (event == null || event.length() == 0) {
			if (defaultEvent == null) {
				throw new EventNotFoundException("No event is specified and no default event is configured for action " + actionClass.getName());
			}
			return defaultEvent;
		}
		Event metadata = events.get(event);
		if (metadata == null) {
			throw new EventNotFoundException("No such event '" + event + "' exists for action " + actionClass.getName());
		}
		return metadata;
	}
	
	/**
	 * @return the propertyBinder bound to a normalized property path
	 * @throws PropertyNotFoundException if the property could not be found
	 */
	public PropertyBinder getPropertyBinder(String normalizedPropertyPath) {
		return getPropertyBinderMap().getRequiredPropertyBinder(normalizedPropertyPath);
	}
	
	/**
	 * Adds a validator to all events that match the validator event filter
	 * @param validator the validator to be added
	 */
	public void addValidator(PathMatcher pathMatcher, PropertyValidator validator) {
		for (Event event : events.values()) {
			if (pathMatcher == null || pathMatcher.matches(event.getName())) {
				event.addValidator(validator);
			}
		}
	}
	
	/**
	 * Remove an event from the list of available events
	 * @param eventName the name of the Event to be removed
	 * @return the removed Event, null if none
	 */
	public Event removeEvent(String eventName) {
		return events.remove(eventName);
	}
	
	/**
	 * @return the converter associated to the provided property, null if none.
	 * This method also returns null if the field is bound to an internal loom value
	 */
	public Converter getConverter(String normalizedPropertyPath) {
		Converter c = null;
		if (!normalizedPropertyPath.startsWith(RequestParameterNames.LOOM_PREFIX)) {
			PropertyBinder propertyBinder = getPropertyBinderMap().getPropertyBinder(normalizedPropertyPath);
			if (propertyBinder != null) {
				c = propertyBinder.getConverter();
			}
		}
		return c;
	}
	
	/**
	 * @return true if this ActionMapping contains one Event with the provided name
	 */
	public boolean isEvent(String eventName) {
		return events.containsKey(eventName);
	}
	
	/**
	 * @return the short name of this action, which is the action class name minus the optional suffix "Action"
	 */
	public String getShortName() {
		return shortName;
	}
	
	/**
	 * @return the long name of this action, which is the action class name 
	 */
	public String getLongName() {
		return actionClass.getName();
	}
	
	public Class<? extends Action> getActionClass() {
		return actionClass;
	}

	public void setActionClass(Class<? extends Action> actionClass) {
		this.actionClass = actionClass;
		this.shortName = StringUtils.removeEnd(actionClass.getSimpleName(), "Action");
	}

	/**
	 * @return the values of the specified property path inside this action instance
	 */
	public String getValueAsString(Action action, String propertyPath, MessagesRepository repository) {
		return propertyBinderMap.getValueAsText(action, propertyPath, repository);
	}

	/**
	 * @return the values of the specified property path inside this action instance
	 */
	public Object getValueAsObject(Action action, String propertyPath, MessagesRepository messagesRepository) {
		return propertyBinderMap.getValue(action, propertyPath);
	}

	/**
	 * This method  revises the slash at the beginning and end of the handle before assigning the field.
	 * @param handle
	 */
	public void setHandle(String handle) {
		handle = StringUtils.removeEnd(handle, "/");
		if (handle.charAt(0) != '/') {
			handle = "/" + handle;
		}
		this.handle = handle;
	}
	
	public Event getDefaultEvent() {
		return defaultEvent;
	}

	public void setDefaultEvent(Event defaultEvent) {
		this.defaultEvent = defaultEvent;
		log.debug("Default event for ", actionClass.getName(), ": ", defaultEvent.getName());
	}

	public String getHandle() {
		return handle;
	}

	public Map<String, Event> getEvents() {
		return events;
	}

	public void setPropertyBinderMap(PropertyBinderMap propertyBinderMap) {
		this.propertyBinderMap = propertyBinderMap;
	}

	public PropertyBinderMap getPropertyBinderMap() {
		return propertyBinderMap;
	}

	public Set<Class<?>> getReloadingClasses() {
		return reloadingClasses;
	}

	public void setEvents(Map<String, Event> events) {
		this.events = events;
	}

	public void setReloadingClasses(Set<Class<?>> reloadingClasses) {
		this.reloadingClasses = reloadingClasses;
	}

	public String getBeanName() {
		return beanName;
	}

	public void setBeanName(String beanName) {
		this.beanName = beanName;
	}

	public void setPropertyAsObject(Action action, String propertyPath, Object value) {
		PropertyBinder propertyBinder = getPropertyBinder(PropertyUtils.normalize(propertyPath));
		propertyBinder.setValue(action, propertyPath, value);
	}

	public String getForwardFolder() {
		return forwardFolder;
	}

	public void setForwardFolder(String forwardFolder) {
		this.forwardFolder = forwardFolder;
	}

}
