package org.loom.mapping;

import java.lang.annotation.Annotation;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.binding.AnnotationContainer;
import org.loom.binding.ItemMetadata;
import org.loom.binding.PropertyBinder;
import org.loom.binding.PropertyBinderFactory;
import org.loom.converter.Converter;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.util.ClassUtils;
import org.loom.util.PropertyUtils;
import org.loom.validator.PropertyValidator;
import org.loom.validator.ValidationRequest;
import org.springframework.core.GenericCollectionTypeResolver;
import org.springframework.core.MethodParameter;

import com.google.common.collect.LinkedHashMultimap;
import com.google.common.collect.SetMultimap;

/**
 * Argument received by an Event
 * @author icoloma
 */
public class EventParameter implements AnnotationContainer {

	/** the argument name */
	private String name;
	
	/** the parameter class */
	private Class<?> parameterType;
	
	/** for simple parameters, the {@link Converter} instance to be used */
	private Converter converter;
	
	/** for complex types, the properties that can be populated */
	private Map<String, PropertyBinder> properties;

	private PropertyBinderFactory propertyBinderFactory;
	
	/** the list of validators to execute for this parameter */
	private SetMultimap<String, PropertyValidator> parameterValidators = LinkedHashMultimap.create();
	
	/** the list of annotations set on this parameter */
	private Annotation[] annotations;
	
	/** the event */
	private Event event;
	
	/** the zero-based parameter index */
	private int index;
	
	/** 
	 * the list of item metadata specified by SetMetadata, CollectionMetadata or MapMetadata. It is handled in the same order as generics parameters.
	 * Lists have a single value that corresponds to the list item type, and Maps have two values that are the key type and the value type, respectively. 
	 */
	private ItemMetadata[] itemMetadata; 

	private static Log log = Log.getLog();
	
	@Override
	public String toString() {
		return new ToStringBuilder(this)
			.append("name", name)
			.append("parameterType", parameterType.getName())
			.append("converter", converter)
			.append("annotations", annotations)
			.toString() ;
	}
	
	/**
	 * Return the list of created parameter values for this request
	 */
	public Object populate(Object container, ServletRequestParameters parameters, MessagesRepository repository, Messages messages) {
		
		// if it's a simple parameter
		if (converter != null) {
			String stringValue = parameters.getStringParameter(name);
			parameters.setAssigned(name);
			return converter.getAsObject(name, stringValue, messages, repository);
		}
		
		// complex objects
		for (Map.Entry<String, String> entry : parameters.getStringParametersEntrySet()) {
			String paramName = entry.getKey();
			if (paramName.startsWith(name)) {
				log.trace("Setting ", paramName);
				if (container == null) {
					container = ClassUtils.newInstance(parameterType);
				}
				String paramValue = entry.getValue();
				PropertyBinder propertyBinder = getPropertyBinder(PropertyUtils.normalize(paramName));
				Object valueAsObject = propertyBinder.getConverter().getAsObject(paramName, paramValue, messages, repository);
				propertyBinder.setValue(container, StringUtils.removeStart(paramName, name + "."), valueAsObject);
				parameters.setAssigned(paramName);
			}
		}
		return container;
	}

	/** 
	 * execute the validations configured for this event parameter
	 * @param container the event parameter value
	 * @param messages where validation errors will be added
	 */
	public void validate(Object container, Messages messages) {
		for (PropertyValidator validator : parameterValidators.values()) {
			String normalizedPropertyPath = validator.getPropertyPath();
			if (!messages.hasAnyError(normalizedPropertyPath)) { // do not overwrite previous (conversion) errors
				log.trace("Validating ", normalizedPropertyPath);
				ValidationRequest validationRequest = new ValidationRequest(validator, messages).push(name);
				if (normalizedPropertyPath.length() == name.length()) { // simple  parameter
					validationRequest.validate(container);
				} else { // composite parameter
					PropertyBinder binder = getPropertyBinder(normalizedPropertyPath);
					binder.applyValidation(container, validationRequest);
				}
			}
		}
	}

	/**
	 * @param normalizedPropertyPathWithPrefix the property path including the parameter name
	 */
	public PropertyBinder getPropertyBinder(String normalizedPropertyPathWithPrefix) {
		String normalizedPropertyPath = normalizedPropertyPathWithPrefix.substring(name.length() + 1);
		PropertyBinder result = properties.get(normalizedPropertyPath);
		if (result != null) {
			return result;
		}
		result = propertyBinderFactory.create(parameterType, normalizedPropertyPath);
		properties.put(normalizedPropertyPath, result);
		return result;
	}

	/**
	 * @return true if this EventParameter handles the proposed property path
	 */
	public boolean matches(String propertyPath) {
		return propertyPath.startsWith(name) && (
				propertyPath.length() == name.length() || 
				properties != null && propertyPath.charAt(name.length()) == '.');
	}
	
	public Object getPropertyAsObject(Object container, String propertyPath) {
		if (propertyPath.length() == name.length()) {
			return container;
		}
		PropertyBinder binder = getPropertyBinder(PropertyUtils.normalize(propertyPath));
		return binder.getValue(container, propertyPath.substring(name.length() + 1));
	}
	
	public String getPropertyAsString(Object container, String propertyPath, MessagesRepository repository) {
		if (propertyPath.length() == name.length()) {
			return converter.getAsText(container, repository);
		}
		PropertyBinder binder = getPropertyBinder(PropertyUtils.normalize(propertyPath));
		return binder.getValueAsText(container, propertyPath.substring(name.length() + 1), repository);

	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Class<?> getParameterType() {
		return parameterType;
	}
	
	/**
	 * this method exists only to comply with the implemented interface {@link AnnotationContainer}
	 */
	public Class getPropertyClass() {
		return getParameterType();
	}

	public void setParameterType(Class<?> parameterType) {
		this.parameterType = parameterType;
	}

	public Converter getConverter() {
		return converter;
	}

	public void setConverter(Converter converter) {
		this.converter = converter;
		if (converter == null) {
			properties = new ConcurrentHashMap<String, PropertyBinder>();
		}
	}
	public void setPropertyBinderFactory(PropertyBinderFactory propertyBinderFactory) {
		this.propertyBinderFactory = propertyBinderFactory;
	}
	
	public void addValidator(PropertyValidator validator) {
		parameterValidators.put(validator.getPropertyPath(), validator);
	}
	
	public Set<PropertyValidator> getValidators(String propertyPath) {
		return parameterValidators.get(propertyPath);
	}

	public void setAnnotations(Annotation[] annotations) {
		this.annotations = annotations;
	}

	public Annotation[] getAnnotations() {
		return annotations;
	}
	
	/**
	 * If the parameter has the provided annotation, return it. Else, return null.
	 */
	@SuppressWarnings("unchecked")
	public <T extends Annotation> T getAnnotation(Class<T> annotationClass) {
		for (Annotation annotation : annotations) {
			if (annotationClass.isAssignableFrom(annotation.getClass())) {
				return (T) annotation;
			}
		}
		return null;
	}

	public String getDisplayName() {
		return "event parameter " + name;
	}

	/**
	 * Assigns value to some property nested inside this eventParameter
	 */
	public void setPropertyAsObject(Map<String, Object> parameterValues, String propertyPath, Object value) {
		if (name.length() == propertyPath.length()) {
			parameterValues.put(propertyPath, value);
		} else {
			Object rootParameterValue = parameterValues.get(name);
			if (rootParameterValue == null && value != null) {
				rootParameterValue = ClassUtils.newInstance(parameterType);
				parameterValues.put(name, rootParameterValue);
			}
			PropertyBinder propertyBinder = getPropertyBinder(PropertyUtils.normalize(propertyPath));
			propertyBinder.setValue(rootParameterValue, propertyPath.substring(name.length() + 1), value);
		}
	}

	public Event getEvent() {
		return event;
	}

	public void setEvent(Event event) {
		this.event = event;
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int index) {
		this.index = index;
	}

	public Class<?> guessCollectionGenericType() {
		return GenericCollectionTypeResolver.getCollectionParameterType(new MethodParameter(event.getJavaMethod(), index));
	}

	public Class<?> guessMapGenericKeyType() {
		return GenericCollectionTypeResolver.getMapKeyParameterType(new MethodParameter(event.getJavaMethod(), index));
	}

	public Class<?> guessMapGenericValueType() {
		return GenericCollectionTypeResolver.getMapValueParameterType(new MethodParameter(event.getJavaMethod(), index));
	}
	
	public boolean isCollection() {
		return Collection.class.isAssignableFrom(parameterType) || parameterType.isArray(); 
	}
	
	public boolean isMap() {
		return Map.class.isAssignableFrom(parameterType); 
	}

	public void setItemMetadata(ItemMetadata[] itemMetadata) {
		this.itemMetadata = itemMetadata;
	}
	
	public ItemMetadata getItemMetadata(int index) {
		return itemMetadata == null?  null : itemMetadata[index];
	}
	
	public int getItemMetadataSize() {
		return itemMetadata == null?  0 : itemMetadata.length;
	}
	
}
