/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.mapping;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;

import org.loom.action.Action;
import org.loom.annotation.processor.AnnotationProcessor;
import org.loom.annotation.processor.AnnotationProcessorRepository;
import org.loom.binding.PropertyBinderFactory;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.converter.Converter;
import org.loom.converter.ConverterFactory;
import org.loom.converter.ConverterRepository;
import org.loom.exception.ConfigException;
import org.loom.log.Log;
import org.loom.resolution.Resolution;
import org.loom.util.ClassUtils;
import org.loom.util.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;


/**
 * Creates a RESTful ActionMapping for an Action 
 * @author icoloma
 *
 */
public class RestActionMappingFactory implements ActionMappingFactory, InitializingBean {

	@Autowired(required=false)
	private PropertyBinderFactory propertyBinderFactory;
	
	@Autowired(required=false)
	private ConverterFactory converterFactory;
	
	/**
	 * processes annotations on classes, translating them to modifications 
	 * on the ActionMapping instance
	 */
	@Autowired(required=false)
	private AnnotationProcessorRepository annotationProcessorRepository;

	private static Log log = Log.getLog();
	
	public void afterPropertiesSet() {
		if (annotationProcessorRepository == null) {
			throw new IllegalArgumentException("No value has been specified for annotationProcessorRepository");
		}
		if (propertyBinderFactory == null) {
			throw new IllegalArgumentException("Must specify either propertyBinderFactory or propertyWrapperFactory");
		}
		if (converterFactory == null) {
			converterFactory = ConverterRepository.getInstance();
		}
	}
	
	public ActionMapping create(String beanName, Class<? extends Action> actionClass) {
		String handle = createHandle(actionClass);
		ActionMapping mapping = new ActionMapping();
		mapping.setBeanName(beanName);
		mapping.setHandle(handle);
		mapping.setForwardFolder(handle);
		mapping.setActionClass(actionClass);
		mapping.setPropertyBinderMap(propertyBinderFactory.createPropertyBinderMap(actionClass));
		
		// read all available events
		initEvents(mapping);
		
		// launch the ActionAnnotationParsers
		annotationProcessorRepository.process(mapping);
		return mapping;
	}

	public void initEvents(ActionMapping mapping) {
		Class<? extends Action> actionClass = mapping.getActionClass();
		Method[] candidateEvents = ClassUtils.findMethodsReturning(actionClass, Resolution.class);
		if (candidateEvents.length == 0) {
			throw new ConfigException("No events found in class " + actionClass.getName());
		}
		
		Map<String, Event> events = new HashMap<String, Event>();
		String[] eventNames = new String[candidateEvents.length];
		for (int i = 0; i < candidateEvents.length; i++) { 
			Method method = candidateEvents[i];
			String name = method.getName();
			eventNames[i] = name; 
			events.put(name, createEvent(mapping, method));
		}
		if (events.size() == 1) {
			mapping.setDefaultEvent(events.values().iterator().next());
		}
		mapping.setEvents(events);
		log.debug("Events for ", actionClass.getName(), ": ", eventNames);
	}

	public String createHandle(Class<? extends Action> actionClass) {
		// removes the Action suffix
		String s = StringUtils.removeEnd(actionClass.getSimpleName(), "Action");
		return "/" + StringUtils.dasherize(s);
	}

	protected Event createEvent(ActionMapping mapping, Method method) {
		Event event = new Event(mapping, method);
		String eventName = method.getName();
		event.setUriParser(UriParser.newInstance("/" + StringUtils.dasherize(eventName)));
		addEventParameters(event);
		return event;
	}

	/**
	 * Add event parameters metadata
	 */
	private void addEventParameters(Event event) {
		Method method = event.getJavaMethod();
		Class<?>[] types = method.getParameterTypes();
		Annotation[][] methodAnnotations = method.getParameterAnnotations();
		for (int i = 0; i < types.length; i++) {
			Class<?> type = types[i];
			EventParameter parameter = new EventParameter();
			parameter.setPropertyBinderFactory(propertyBinderFactory);
			parameter.setParameterType(type);
			parameter.setAnnotations(methodAnnotations[i]);
			parameter.setEvent(event);
			parameter.setIndex(i);
			parameter.setItemMetadata(propertyBinderFactory.getPropertyWrapperFactory().createItemMetadata(parameter));
			Converter converter = converterFactory.getConverter(parameter);
			parameter.setConverter(converter);
			event.addEventParameter(parameter);
		}
	}

	public void addAnnotationProcessor(AnnotationProcessor processor) {
		annotationProcessorRepository.addAnnotationProcessor(processor);
	}

	public SortedSet<AnnotationProcessor> getAnnotationProcessors() {
		return annotationProcessorRepository.getAnnotationProcessors();
	}

	public void setPropertyBinderFactory(PropertyBinderFactory propertyBinderFactory) {
		this.propertyBinderFactory = propertyBinderFactory;
	}

	public void setAnnotationProcessorRepository(
			AnnotationProcessorRepository annotationProcessorRepository) {
		this.annotationProcessorRepository = annotationProcessorRepository;
	}

	public void setConverterFactory(ConverterFactory converterFactory) {
		this.converterFactory = converterFactory;
	}
	
	@Autowired(required=false)
	public void setPropertyWrapperFactory(PropertyWrapperFactory factory) {
		this.propertyBinderFactory = new PropertyBinderFactory();
		propertyBinderFactory.setPropertyWrapperFactory(factory);
	}

	public PropertyBinderFactory getPropertyBinderFactory() {
		return propertyBinderFactory;
	}

	public ConverterFactory getConverterFactory() {
		return converterFactory;
	}

	public AnnotationProcessorRepository getAnnotationProcessorRepository() {
		return annotationProcessorRepository;
	}
	
	
}
