package org.loom.paged;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Criteria of a requested page
 * @author icoloma
 *
 */
public class PagedListCriteriaImpl extends AbstractPagedContainer implements PagedListCriteria {

	/** the JPA query */
	private String query;
	
	/** the query, after adding the ORDER-BY clause (lazily instantiated) */
	private String orderedQuery;
	
	/** the list of parameters to use in the query */
	private List<Object> parameters = new ArrayList<Object>();
	
	/** true to calculate the totalnumber of rows, default true */
	private boolean calculateTotal = true;
	
	public PagedListCriteriaImpl() {
	}
	
	/**
	 * Add a parameter to the query
	 * @param paramValue the value of the parameter 
	 */
	public PagedListCriteriaImpl addParameter(Object paramValue) {
		parameters.add(paramValue);
		return this;
	}
	
	/**
	 * @return the user-specified query, after adding the ORDER BY clause specified by sortProperty and sortOrder
	 */
	public String getOrderedQuery() {
		if (orderedQuery == null) {
			orderedQuery = addOrderBy(query);
		}
		return orderedQuery;
	}

	/**
	 * Set the query to be executed, and any additional parameter as well.
	 * The params can also be set latter by invoking addParameter()
	 * @param query the JPA query to execute
	 * @param params the parameters to add to the query, in the expected order
	 */
	public PagedListCriteriaImpl setQuery(String query, Object... params) {
		this.query = query;
		this.orderedQuery = null;
		if (params != null) {
			Collections.addAll(parameters, params);
		}
		return this;
	}

	/**
	 * Adds the order by parameter to a query. If there is already an
	 * order by, <b>prepends</b> the current order to the one included
	 * in the query
	 */
	public String addOrderBy(String query) {
		// no order by
		if (getSortProperty() == null) {
			return query;
		}
		int pos = locateOrderBy(query);
		if (pos == -1) {
			// order by one field
			return query + " order by " + getSortProperty() + " " + getSortOrder().getValue();
		} else {
			// order by multiple fields
			return query.substring(0, pos) +  getSortProperty() + " " + getSortOrder().getValue() + ", " + query.substring(pos);
		}
	}

	/**
	 * Locates the "order by" clause, if there is one
	 * @param query the QL query to inspect
	 * @return The position of the first character after the "order by" clause if there is one, -1 if not.
	 */
	private int locateOrderBy(String query) {
		int pos = query.lastIndexOf(" order ");
		if (pos != -1) {
			pos = query.lastIndexOf("by ", pos + 7);
			if (pos != -1) {
				return pos + 3;
			}
		}
		return pos;
	}

	public boolean calculateTotal() {
		return calculateTotal;
	}
	
	public boolean isCalculateTotal() {
		return calculateTotal;
	}

	public void setCalculateTotal(boolean calculateTotal) {
		this.calculateTotal = calculateTotal;
	}

	public List<Object> getParameters() {
		return parameters;
	}
	
	@Override
	public PagedListCriteriaImpl setId(String id) {
		return (PagedListCriteriaImpl) super.setId(id);
	}
	
}
