package org.loom.paged;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import com.google.common.base.Function;

public class PagedListDataFactory {

	/**
	 * Creates a data object according to the provided criteria
	 * @param criteria the criteria object, where page index, sorting etc will be extracted from. 
	 * @return a {@link PagedListData} instance with the same properties of the provided criteria object
	 */
	public static <T> PagedListData<T> create(PagedListCriteria criteria) {
		PagedListDataImpl<T> data = new PagedListDataImpl<T>();
		data.setPageIndex(criteria.getPageIndex());
		data.setPageSize(criteria.getPageSize());
		data.setSortOrder(criteria.getSortOrder());
		data.setSortProperty(criteria.getSortProperty());
		data.setId(criteria.getId());
		return data;
	}
	
	/**
	 * Creates a data object 
	 * Note that this method is (relatively) inefficient and sorting and paging should be done by the backend instead.
	 * @param criteria the criteria object, where page index, sorting etc will be extracted from. 
	 * @param list the entire list of results. It will be sorted and the current page will be extracted from it.
	 * @return a {@link PagedListData} instance with the same properties of the provided criteria object, and the 
	 * contents of the current page extracted from the provided list.
	 */
	@SuppressWarnings("unchecked")
	public static <T> PagedListData<T> create(PagedListCriteria criteria, List<T> list) {
		Comparator<T> comparator = null;
		if (!list.isEmpty() && criteria.getSortProperty() != null) {
			comparator = new PagedListItemComparator<T>((Class<T>)list.get(0).getClass(), criteria.getSortProperty(), criteria.getSortOrder());
		}
		return create(list, comparator, criteria);
	}
	
	/**
	 * Creates a data object according to the provided criteria
	 * 
	 * @param <T> the type of each record object
	 * @param list the entire list of results. Pagination and ordering will be applied on this.
	 * @param comparator the Comparator to do the sorting. If null, no sort will be performed
	 * @param criteria The pagination,  sorting properties, etc.
	 * @return a PagedListData instance with the same properties as the provided criteria object,
	 * and the list of rows corresponding to this page.
	 */
	@SuppressWarnings("unchecked")
	public static <T> PagedListData<T> create(List<T> list, Comparator<T> comparator, PagedListCriteria criteria) {
		PagedListDataImpl<T> data = (PagedListDataImpl<T>) create(criteria);
		data.setTotalRecords(list.size());
		
		// sort, if needed
		if (comparator != null) {
			Collections.sort(list, comparator);
		}
		
		// extract the current page
		int max = Math.min(list.size(), criteria.getRecordIndex() + criteria.getPageSize());
		List<T> sublist = list.subList(criteria.getRecordIndex(), max);
		data.setData(sublist);
		return data;
	}
	
	/**
	 * Convert the data of one PagedListData into another PagedListData instance, transforming the list contents.
	 * The resulting instance will have the same pageIndex, size, total results, etc, but its data will
	 * be the result of applying the function to each item in source.getData() 
	 * @param <T1> The type of the source list contents
	 * @param <T2> The type of the destination list contents
	 * @param sourceData the PagedListData that will be transformed 
	 * @param function the function that will transform each item
	 * @return A PagedListData instance with the result of applying the function to the source object.
	 */
	@SuppressWarnings("unchecked")
	public static <T1, T2> PagedListData<T2> convert(PagedListData<T1> sourceData, Function<T1, T2> function) {
		PagedListDataImpl<T2> result = ((PagedListDataImpl<T2>) sourceData).clone();
		List<T1> sourceList = sourceData.getData();
		List<T2> copiedList = new ArrayList<T2>(sourceList.size());
		for (T1 t1 : sourceList) {
			copiedList.add(function.apply(t1));
		}
		result.setData(copiedList);
		return result;
	}
	
}
