/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;

import org.apache.commons.lang.StringUtils;
import org.loom.json.JsonMarshaller;
import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.names.ResponseHeaderNames;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.SetMultimap;

/**
 * A response object with http headers. 
 * 
 * @author Ignacio Coloma
 */
public abstract class AbstractHttpResolution implements CacheableResolution {

	/** the Content-Type header, without charset */
	private String contentType;
	
	/** the charset part of the Content-Type header.  */
	private String charset;
	
	/** list of http headers */
	private SetMultimap<String, HttpHeader> headers = HashMultimap.create();

	/** list of cookies */
	private List<Cookie> cookies = new ArrayList<Cookie>();

	/** browser caching data */
	private CacheControl cacheControl;
	
	public abstract void resolve(LoomServletRequest request, LoomServletResponse response) throws IOException, ServletException;

	/**
	 * Adds http headers to the response before resolving
	 */
	protected void writeHeaders(LoomServletRequest request, LoomServletResponse response) {

		// assign contentType and encoding
		if (contentType != null) {
			response.setContentType(contentType);
		}
		if (charset != null) {
			response.setCharacterEncoding(charset);
		}
		
		// add cacheControl headers
		if (cacheControl != null) {
			cacheControl.writeHeaders(response);
		}
		
		// add http headers
		for (HttpHeader header : headers.values()) {
			header.write(response);
		}
		
		// add cookies
		for (Cookie cookie : cookies) {
			response.addCookie(cookie);
		}
	}
	
	/**
	 * Adds a cookie that will not be stored on disk (will be deleted when the browser is closed). 
	 * @param name the name of the cookie
	 * @param value the value of the Cookie
	 */
	public void addCookie(String name, String value) {
		addCookie(name, value, -1);
	}

	/**
	 * Adds a cookie  
	 * @param name the name of the cookie
	 * @param value the value of the Cookie
	 * @param maxAge
	 *            can be positive (number of seconds that it will be stored in
	 *            the browser), zero (to delete the cookie from the browser) or
	 *            a negative value (to indicate that the cookie should not be
	 *            stored in disk and will be forgotten when the browser exists).
	 */
	public void addCookie(String name, String value, int maxAge) {
		Cookie cookie = new Cookie(name, value);
		cookie.setMaxAge(maxAge);
		cookies.add(cookie);
	}

	/**
	 * Adds a cookie
	 * @param cookie the cookie to be added to the response
	 */
	public void addCookie(Cookie cookie) {
		cookies.add(cookie);
	}

	private void addHeader(HttpHeader header) {
		headers.put(header.getName(), header);
	}
	
	/**
	 * Add a string http header. If the header already exists a new value will be added.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void addHeader(String name, String value) {
		addHeader(new StringHttpHeader(name, value, true));
	}

	/**
	 * Add a date http header. If the header already exists a new value will be added.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void addDateHeader(String name, Date value) {
		addHeader(new DateHttpHeader(name, value, true));
	}

	/**
	 * Add an integer http header. If the header already exists a new value will be added.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void addIntHeader(String name, int value) {
		addHeader(new IntHttpHeader(name, value, true));
	}

	/**
	 * Set a String http header. If the header already exists it will be replaced.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void setHeader(String name, String value) {
		addHeader(new StringHttpHeader(name, value, false));
	}

	/**
	 * Set a String http header if it has not been set.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void setHeaderIfNotPresent(String name, String value) {
		if (!containsHeader(name)) {
			setHeader(name, value);
		}
	}

	/**
	 * Set a date http header. If the header already exists it will be replaced.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void setDateHeader(String name, Date value) {
		addHeader(new DateHttpHeader(name, value, false));
	}

	/**
	 * Set a Date http header if it has not been set.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void setDateHeaderIfNotPresent(String name, Date value) {
		if (!containsHeader(name)) {
			setDateHeader(name, value);
		}
	}

	/**
	 * Set an integer http header. If the header already exists it will be replaced.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void setIntHeader(String name, int value) {
		addHeader(new IntHttpHeader(name, value, false));
	}

	/**
	 * Set an integer http header if it's not already set.
	 * @param name the name of the http header
	 * @param value the value of the http header
	 */
	public void setIntHeaderIfNotPresent(String name, int value) {
		if (!containsHeader(name)) {
			setIntHeader(name, value);
		}
	}
	
	/**
	 * sets the X-JSON http header
	 * @param marshaller the JSON object to be added as http header
	 * @deprecated use jackson instead
	 */
	public void setJsonHeader(JsonMarshaller marshaller) {
		// newlines are not permitted in http headers
		String jsonString = StringUtils.replaceChars(marshaller.toString(), "\n\r", "");
		if (jsonString.length() > 8 * 1024) {
			throw new IllegalArgumentException("Json object is bigger (" + jsonString.length() + " bytes) than the maximum limit of 8k. Consider using JsonResolution with Transport.AS_HTML_BODY instead.");
		}
		setHeader(ResponseHeaderNames.X_JSON, jsonString); 
	}

	/**
	 * @param name the http header name to check
	 * @return true if there is a http header with the provided name
	 */
	public boolean containsHeader(String name) {
		return headers.containsKey(name);
	}

	public CacheControl getCacheControl() {
		return cacheControl;
	}

	public AbstractHttpResolution setCacheControl(CacheControl cacheControl) {
		this.cacheControl = cacheControl;
		return this;
	}

	public String getContentType() {
		return contentType;
	}

	public AbstractHttpResolution setContentType(String contentType) {
		this.contentType = contentType;
		return this;
	}

	public String getCharset() {
		return charset;
	}

	public AbstractHttpResolution setCharset(String charset) {
		this.charset = charset;
		return this;
	}

}
