/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.servlet.ServletException;

import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.util.HtmlSanitizer;

/**
 * Returns an unordered list of autocompleter results highlighting the 
 * match part in every result.
 * 
 * @author Rafael Serrano
 */
public class AutocompleteResolution extends AbstractHttpResolution {

	/** The list containing all the highlighted entries */
	private List<AutocompleteEntry> entries = new ArrayList<AutocompleteEntry>();

	/** The text to highlight in every entry */
	private String highlightText;
	
	/** the regex used to highlight text in the results list */
	private Pattern highlightPattern;
	
	/** true to remove HTML tags of each result value, default true */ 
	private boolean sanitizeValues = true;

	public AutocompleteResolution(String highlightText) {
		this(Pattern.compile(Pattern.quote(highlightText != null ? highlightText : ""), Pattern.CASE_INSENSITIVE), highlightText);
	}
	
	public AutocompleteResolution(Pattern highlightPattern, String highlightText) {
		this.highlightPattern = highlightPattern;
		this.highlightText = highlightText;
	}

	/** Adds a new item to the results list */
	public AutocompleteEntry add(String value) {
		AutocompleteEntry entry = new AutocompleteEntry(value);
		entries.add(entry);
		return entry;
	}
	
	/** Adds every String inside a Collection */ 
	public void addAll(Iterable<String> strings) {
		for (String s : strings) {
			add(s);
		}
	}

	@Override
	public void resolve(LoomServletRequest request, LoomServletResponse response)
			throws IOException, ServletException {
		response.setContentType("text/html");
		writeHeaders(request, response);
		StringBuilder htmlList = new StringBuilder(entries.size() * 40);
		htmlList.append("<ul>");
		for (AutocompleteEntry entry : entries) {
			htmlList.append("<li");
			
			// append the list of attributes
			Map<String, Object> attributes = entry.getAttributes();
			if (attributes != null) {
				for (Map.Entry<String, Object> me : attributes.entrySet()) {
					if (me.getValue() != null) {
						String value = HtmlSanitizer.sanitize(me.getValue().toString());
						htmlList.append(" ").append(me.getKey()).append("=\"").append(value).append("\"");
					}
				}
			}
			htmlList.append(">");
			String value = sanitizeValues? HtmlSanitizer.sanitize(entry.getValue()) : entry.getValue();
			htmlList.append(highlight(value));
			htmlList.append("</li>");
		}
		htmlList.append("</ul>");
		response.getWriter().write(htmlList.toString());
	}

	protected String highlight(String text) {
		return highlightPattern.matcher(text).replaceAll("<strong>$0</strong>");
	}

	public String getHighlightText() {
		return highlightText;
	}

	public Pattern getHighlightPattern() {
		return highlightPattern;
	}

	public boolean isSanitizeValues() {
		return sanitizeValues;
	}

	public void setSanitizeValues(boolean sanitizeValues) {
		this.sanitizeValues = sanitizeValues;
	}
	
}
