/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.ArrayUtils;
import org.loom.action.Action;
import org.loom.log.Log;
import org.loom.mapping.ParsedAction;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.names.ResponseHeaderNames;
import org.loom.url.UrlBuilder;


/**
 * Redirect resolution implementation.
 * 
 * @author Ignacio Coloma
 */
public class RedirectResolutionImpl extends AbstractHttpResolution implements RedirectResolution {

	/** location and parameters (name, value) where to redirect */
	private UrlBuilder url;
	
	/** list of properties to append */
	private List<String> properties;
	
	/** true to return a permanent redirect (301), false for temporary (302). Default is false */
	private boolean permanentFlag;
	
	private static Log log = Log.getLog();
	
	public RedirectResolutionImpl(String url) {
		this.url = new UrlBuilder();
		this.url.setRawUrl(url);
	}
	
	public RedirectResolutionImpl(String action, String event) {
		this.url = new UrlBuilder();
		this.url.setAction(action);
		this.url.setEvent(event);
	}
	
	public RedirectResolutionImpl(Class<? extends Action> actionClass, String event) {
		this(actionClass.getName(), event);
	}
	
	public RedirectResolution add(String name, Object value) {
		url.add(name, value);
		return this;
	}
	
	public RedirectResolution addParameter(String name, Object value) {
		return add(name, value);
	}
	
	public RedirectResolution addProperty(String property) {
		if (properties == null) {
			properties = new ArrayList<String>();
		}
		properties.add(property);
		return this;
	}
	
	public RedirectResolution addPropertyParameter(String propertyName) {
		return addProperty(propertyName);
	}
	
	public RedirectResolution setHash(String hash) {
		url.setHash(hash);
		return this;
	}

	@Override
	public void resolve(LoomServletRequest request, LoomServletResponse response) throws IOException {
		writeHeaders(request, response);
		ParsedAction parsedAction = request.getParsedAction();
		if (properties != null) {
			if (parsedAction == null) {
				throw new IllegalArgumentException("No source action trying to get properties: " + ArrayUtils.toString(properties.toArray()) + " to append to redirect to url: " + url);
			}
			for (String property : properties) {
				url.add(property, parsedAction.getPropertyAsString(property));
			}
		}
		if (parsedAction != null) {
			parsedAction.getMessages().saveForRedirect(request);
		}
		
		String destination = url.getURL();
		
		// LOOM-37: IE 6 does not handle anchors properly when redirecting
		if (url.getHash() != null && !url.getParameters().isEmpty() && request.getBrowserData().isIE() && request.getBrowserData().getVersion().compareTo("7") < 0) {
			// add a trailing '&' before the anchor
			destination = destination.substring(0, destination.length() - url.getHash().length() - 1) + "&#" + url.getHash();
		}
		
		log.debug("Redirecting request to ", destination);
		response.setStatus(permanentFlag? HttpServletResponse.SC_MOVED_PERMANENTLY : HttpServletResponse.SC_MOVED_TEMPORARILY);
		response.setHeader(ResponseHeaderNames.LOCATION, destination);
	}
	
	public RedirectResolution permanent() {
		this.permanentFlag = true;
		return this;
	}
	
	public boolean isPermanent() {
		return permanentFlag;
	}

	public List<String> remove(String name) {
		return url.remove(name);
	}
	
	public List<String> removeParameter(String name) {
		return remove(name);
	}

	public List<String> replace(String name, Object value) {
		return url.replace(name, value);
	}
	
	public List<String> replaceParameter(String name, Object value) {
		return replace(name, value);
	}

	public RedirectResolution setScheme(String scheme) {
		url.setScheme(scheme);
		return this;
	}

	public RedirectResolution setHostname(String hostname) {
		url.setHostname(hostname);
		return this;
	}

	public RedirectResolution setPort(Integer port) {
		url.setPort(port);
		return this;
	}

	public RedirectResolution setIncludeRequestParameters( boolean addCurrentRequestParameters) {
		url.setIncludeRequestParameters(addCurrentRequestParameters);
		return this;
	}
	
}
