/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import org.loom.action.Action;
import org.loom.persistence.file.PersistentFile;

/**
 * A factory for Resolution classes.
 * TODO This class may be redesigned in the future
 * 
 * @author icoloma
 */
public interface ResolutionFactory {

	/**
	 * Create a ForwardResolution
	 * @param pageName the name of the forward destination
	 * @return a ForwardResolution to the provided resource
	 */
	public ForwardResolution forward(String pageName);
	
	/**
	 * Forward to a page, with preference for a localized page if possible.
	 * A forward to "myfile.jsp" with spanish request locale and english default locale 
	 * will first try "myfile_es.jsp", then "myfile_en.jsp" and, if none is found, "myfile.jsp"
	 * @see LocaleForwardResolutionImpl
	 * @param pageName the name of the resource to forward to
	 * @return a ForwardResolution to the provided page
	 */
	public ForwardResolution forwardWithLocale(String pageName);
	
	/**
	 * Sends a persistent file to the browser.
	 */
	public PersistentFileResolution send(PersistentFile file);
	
	/**
	 * Redirects the browser to a different URL
	 * @param url the url for the browser redirection
	 * @return a {@link RedirectResolution} pointing to the provided URL 
	 */
	public RedirectResolution redirect(String url);
	
	/**
	 * Redirects the browser to a different Action and Event
	 * @param action the action short or long name 
	 * @param event the event to be invoked, null for default event
	 * @return a {@link RedirectResolution} pointing to the provided URL 
	 */
	public RedirectResolution redirect(String action, String event);
	
	/**
	 * Redirects the browser to a different Action and Event
	 * @param actionClass the action class
	 * @param event the event to be called, null for default event
	 * @return a {@link RedirectResolution} pointing to the provided URL 
	 */
	public RedirectResolution redirect(Class<? extends Action> actionClass, String event);

	/**
	 * Sends a JSON serialization of the contents object. 
	 * @param contents the contents object to serialize as a JSON object
	 * @return the JSON resolution 
	 */
	public JacksonResolution json(Object contents);
	
}
