/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet;

import static org.loom.servlet.names.RequestAttributeNames.FLASH_NEXT;
import static org.loom.servlet.names.RequestAttributeNames.FLASH_PREV;
import static org.loom.servlet.names.RequestParameterNames.FLASH_ID;
import static org.loom.servlet.names.SessionAttributeNames.FLASH_PREFIX;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import java.util.concurrent.atomic.AtomicLong;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.loom.util.StringUtils;

/**
 * Contains objects stored in a flash scope.
 * A flash scope is like  a request scope that is alive for two consecutive requests.
 * Objects stored in this context will be available for the next request.
 * The FlashContainer object is Serializable, but it does not enforce that its contents 
 * are Serializable (adopting the same approach of 7.7.2 of the servlet 2.3 spec).
 * Note that this could be a problem when including not Serializable objects in a flash 
 * context, if the server decides to serialize the session.
 * 
 * @author icoloma
 */
public class FlashContainer implements Serializable {

	/** this flash scope ID */
	private String id;
	
	/** objects stored in flash scope */
	private Map<String, Object> contents = new HashMap<String, Object>();

    private static final long serialVersionUID = -2229794470754667710L;
	
    /** generates the identifiers for flash contexts */
    private static AtomicLong idGenerator = new AtomicLong();
    
	private FlashContainer() {
	}
	
	/**
	 * Return the flash-scoped container with objects persisted for the NEXT browser request.
	 * @param create if true, create the flash context if it does not yet exist.
	 * @param request the current NEXT flash container object
	 */
	public static FlashContainer getNext(HttpServletRequest request, boolean create) {
		
		FlashContainer instance = (FlashContainer) request.getAttribute(FLASH_NEXT);
			
		if (instance == null && create) {
			HttpSession session = request.getSession(true);
			synchronized (session) {
				instance = (FlashContainer) request.getAttribute(FLASH_NEXT);
				if (instance == null) {
					instance = new FlashContainer(); 
					String id = StringUtils.numberToAlpha(idGenerator.getAndIncrement());
					instance.setId(id);
					session.setAttribute(FLASH_PREFIX + id, instance);
				}
				request.setAttribute(FLASH_NEXT, instance);
			}
		}
		return instance;
	}
	
	/**
	 * Return the flash-scoped container with objects persisted from the PREVIOUS browser request.
	 * 
	 * @param request the current PREVIOUS flash container, null if none.
	 */
	public static FlashContainer getPrev(HttpServletRequest request) {
		
		FlashContainer instance = (FlashContainer) request.getAttribute(FLASH_PREV);
		
		if (instance == null) {
			String flashID = request.getParameter(FLASH_ID);
			if (flashID !=  null) {
				HttpSession session = request.getSession(false);
				if (session != null) {
					String flashAttributeId = FLASH_PREFIX + flashID;
					instance = (FlashContainer) session.getAttribute(flashAttributeId);
					session.removeAttribute(flashAttributeId);
					request.setAttribute(FLASH_PREV, instance);
				}
			}
		}
		return instance;
	}
	
	public void setAttribute(String key, Object value) {
		contents.put(key, value);
	}
	
	public Object getAttribute(String key) {
		return contents.get(key);
	}
	
	public boolean contains(String key) {
		return contents.containsKey(key);
	}

	public Set<Entry<String, Object>> entrySet() {
		return contents.entrySet();
	}

	public boolean isEmpty() {
		return contents.isEmpty();
	}

	public Set<String> keySet() {
		return contents.keySet();
	}

	public int size() {
		return contents.size();
	}

	public String getId() {
		return id;
	}

	private void setId(String id) {
		this.id = id;
	}

	public void removeAttribute(String name) {
		contents.remove(name);
	}

}
