/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import static org.loom.servlet.names.CssClassNames.ERROR;

import java.io.IOException;
import java.util.Set;

import javax.servlet.jsp.JspException;

import org.loom.converter.Converter;
import org.loom.mapping.ParsedAction;
import org.loom.servlet.names.HtmlExtendedAttributeNames;
import org.loom.util.StringUtils;
import org.loom.validator.PropertyValidator;
import org.tldgen.annotations.Attribute;


/**
 * Parent class for any form field associated to a form input parameter.
 * Docs about the HTML fields can be found <a
 * href="http://www.w3.org/TR/html4/interact/forms.html#edef-INPUT">here</a>
 * 
 * @author icoloma
 */
public abstract class AbstractFormInputTag extends AbstractFormFieldTag implements FormInputTag {
	
	public enum LabelPosition {
		LEFT, RIGHT
	}
	
	/** true to render the error message as a separate HTML component, default null */
	@Attribute
	private Boolean renderErrorDisplayComponent;
	
	/** true to render label, false otherwise. By default it will use the value specified in the container form */
	@Attribute
	private Boolean renderLabel;
	
	/** true to translate label before rendering, false otherwise. defaults to true */
	@Attribute
	private boolean translateLabel = true;
	
	/** when specified, use this value instead of the action property value */
	@Attribute
	private String value;
	
	/** true to render the field just as text (no form field), default false */
	@Attribute
	private Boolean renderAsText;
	
	/** true to sanitize HTML characters when rendering as text, default true */
	@Attribute 
	private boolean escapeHTML = true;
	
	/** list of validators, lazily initialized */
	private Set<PropertyValidator> validators;
	
	/** lazily initialized, true if this component has an associated error */
	private Boolean hasError;
	
	/** 
	 * position of the label of this field. Valid values are left and right. Defaults to left for 
	 * normal fields and right for checkboxes and radio buttons
	 */
	private LabelPosition labelPosition = LabelPosition.LEFT;
	
	/** 
	 * true to add translated name as an extended attribute. Set to false if the translated name
	 * has been already added, for example with a surrounding label.
	 * Default true. 
	 */
	private boolean addTranslatedName = true;
	
	private CssBuilder labelClass;
	
	/** true if this field is bound to a java property, defaults to true */
	@Attribute
	private boolean bound = true;
	
	@Override
	protected void initTag() {
		super.initTag();
		
		// if it's marked as readOnly and no decorator has been set, use read-only
		if (shouldRenderAsText() && decorator == null) {
			decorator = "read-only";
		}
		if (hasError()) { 
			addCssClass(ERROR);
		}
		
		if (bound) {
			String normalizedName = getNormalizedName();
			if (normalizedName != null) {
				
				// add validation attributes (min, max, etc)
				for (PropertyValidator validator : getForm().getTargetEvent().getValidatorsFor(normalizedName)) {
					validator.decorate(this);
				}
				
				// add converter CSS classes (number, date, etc)
				Converter converter = getForm().getTargetEvent().getConverter(normalizedName);
				if (converter != null) { 
					converter.decorate(this);
				}
			}
		}
	}
	
	@Override
	protected void printParentAttributes() throws IOException, JspException {
		// if the default decorator has already included the translated name as a 
		// surrounding label, do nothing.
		// else, add the translated name for the javascript code to display error messages
		if (addTranslatedName) {
			setExtendedAttribute(HtmlExtendedAttributeNames.TRANSLATED_NAME, getTranslatedLabel());
		}
		super.printParentAttributes();
	}

	/**
	 * @return if translateLabel == false, the label as is. Else, the result of calling
	 * MessagesRepository.guessString(label)
	 */
	public String getTranslatedLabel() {
		String l = getLabel(); // allow for lazy initialization
		return translateLabel? repository.guessString(l) : l;
	}

	/**
	 * Calculates this tag value.
	 * If this tag has an associated error  message, return the request value prior to conversion.
	 * Else, extract the property value and convert to String.
	 */
	public String getValue() {
		if (hasError()) {
			return request.getStringParameter(getName());
		} else {
			// else, return the property binding value
			return getPropertyValueAsString();
		}
	}
	
	/**
	 * Gets this item value as String, to show when renderAsText is true
	 * By default, it returns the tag value
	 */
	public String renderAsText() {
		return getValue();
	}
	
	/**
	 * Calculate the current property value. If the source action is not the same as the target action of this form, returns null.
	 * This is supposed to work when we revisit the same form, presumably as validation errors arise from user input.
	 * If the tag has specified a value attribute, it will be returned.
	 * @return the value extracted from the nested property inside the action bean, null if cannot be calculated.
	 */
	private String getPropertyValueAsString() {
		if (value != null) {
			return value;
		}
		
		// calculate the value only if the target action is the same as the parsed action
		ParsedAction parsedAction = request.getParsedAction();
		if (parsedAction != null && parsedAction.getActionMapping().getPropertyBinderMap().getPropertyBinder(getNormalizedName()) != null) {
			return parsedAction.getPropertyAsString(getName());
		} else {
			return null;
		}
	}
	
	/**
	 * @return the value extracted from the nested property inside the action bean.
	 * Note that this method does not consider validation or conversion errors. Consider using 
	 * getValue() instead.
	 */
	protected Object getValueAsObject() {
		ParsedAction parsedAction = request.getParsedAction();
		if (parsedAction == null) {
			return null;
		}
		return parsedAction.getPropertyAsObject(getName()); 
	}
	
	/**
	 * @return true if this component has any validation error associated
	 */
	public boolean hasError() {
		if (hasError == null) {
			ParsedAction parsedAction = request.getParsedAction();
			hasError = parsedAction != null && parsedAction.hasAnyError(getName());
		}
		return hasError;
	}
	
	/**
	 * Lazily initializes the list of Validators asociated to this field.
	 * @return The validator list, empty if none.
	 */
	protected Set<PropertyValidator> getValidators() {
		if (validators == null) {
			validators = getForm().getTargetEvent().getValidatorsFor(getNormalizedName()); 
		}
		return validators;
	}

	/**
	 * As of #LOOM-95, modified to propagate all the applied CSS classes to the surrounding label
	 * @return the css classes to apply to the surrounding label
	 */
	public String getLabelCssClass() {
		return labelClass != null? labelClass.addAll(getCssBuilder()).getCss() : getCssClass();
	}

	public void setRenderLabel(Boolean renderLabel) {
		this.renderLabel = renderLabel;
	}

	
	public void setRenderErrorDisplayComponent(Boolean renderError) {
		this.renderErrorDisplayComponent = renderError;
	}
	
	/**
	 * @return true if the component or the form are set to renderLabel
	 */
	public boolean shouldRenderLabel() {
		return renderLabel != null? renderLabel.booleanValue() : getForm().doRenderLabel();
	}
	
	/**
	 * @return true if the component or the form are set to renderAsText
	 */
	public boolean shouldRenderAsText() {
		return renderAsText != null? renderAsText.booleanValue() : getForm().isRenderAsText();
	}

	/**
	 * @return true if the component or the form are set to renderError display component
	 */
	public boolean shouldRenderError() {
		return renderErrorDisplayComponent != null? renderErrorDisplayComponent.booleanValue() : getForm().doRenderSeparateErrors();
	}

	public void setValue(String value) {
		this.value = value;
	}

	public LabelPosition getLabelPosition() {
		return labelPosition;
	}

	/** 
	 * Position of the label for this field. Valid values are "left" (default for almost all field types) 
	 * and "right" (default for checkboxes and radio buttons)
	 */
	@Attribute
	public void setLabelPos(String labelPosition) {
		try {
			this.labelPosition = LabelPosition.valueOf(labelPosition.toUpperCase());
		} catch (IllegalArgumentException e) {
			throw new IllegalArgumentException("Unknown labelPos '" + labelPosition + "'. Valid values are 'left', 'right'");
		}
	}
	
	protected void setLabelPosition(LabelPosition position) {
		this.labelPosition = position;
	}

	public void setRenderAsText(Boolean renderAsText) {
		this.renderAsText = renderAsText;
	}

	public boolean isTranslateLabel() {
		return translateLabel;
	}

	public void setTranslateLabel(boolean translateLabel) {
		this.translateLabel = translateLabel;
	}

	public void setAddTranslatedName(boolean addTranslatedName) {
		this.addTranslatedName = addTranslatedName;
	}

	public boolean isEscapeHTML() {
		return escapeHTML;
	}

	public void setEscapeHTML(boolean escapeRenderAsText) {
		this.escapeHTML = escapeRenderAsText;
	}

	protected Boolean getRenderLabel() {
		return renderLabel;
	}

	protected Boolean getRenderAsText() {
		return renderAsText;
	}

	/** 
	 * CSS styles to be applied to the surrounding label but not to the input component.
	 * This is useful when using grid systems such as 960gs or YUI-Grid 
	 */
	@Attribute
	public void setLabelClass(String labelClass) {
		this.labelClass = new CssBuilder(StringUtils.split(labelClass, ' '));
	}

	public boolean isBound() {
		return bound;
	}

	public void setBound(boolean bound) {
		this.bound = bound;
	}

}
