/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.tldgen.annotations.Attribute;
import org.loom.util.HtmlSanitizer;

/**
 * Parent class for any HTML tag 
 * @author icoloma
 */
public abstract class AbstractHtmlTag extends AbstractDecoratedTag {
	
	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-accesskey">HTML attribute</a> */
	@Attribute
	private String accesskey;
	
	/** <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a> */
	private CssBuilder cssBuilder = new CssBuilder();
	
	/** false if automatic CSS classes have not yet been added */
	private boolean cssProcessed;
	
	/** <a href="http://www.w3.org/TR/html4/struct/dirlang.html#adef-dir">HTML attribute</a> */
	@Attribute
	private String dir;
	
	/** <a href="http://www.w3.org/TR/html4/struct/global.html#adef-id">HTML attribute</a> */
	@Attribute
	private String id;
	
	/** <a href="http://www.w3.org/TR/html4/struct/dirlang.html#adef-lang">HTML attribute</a> */
	@Attribute
	private String lang;
	
	private String name;
	
	/** <a href="http://www.w3.org/TR/html4/present/styles.html#adef-style">HTML attribute</a> */
	@Attribute
	private String style;

	/** <a href="http://www.w3.org/TR/html4/struct/global.html#adef-title">HTML attribute</a> */
	@Attribute
	private String title;
	
	/** set to true to translate title, default true */
	@Attribute
	private boolean translateTitle = true;

	/** true to execute this tag, default true */
	private boolean ifClause = true;

	@Override
	protected boolean renderTag() {
		return ifClause;
	}
	
	/**
	 * Prints this instance attributes
	 * @throws JspException 
	 */
	@Override
	protected void printParentAttributes() throws IOException, JspException {
		// the two important ones first
		out.printAttribute("id", id);
		out.printAttribute("name", name);
		super.printParentAttributes();
		
		out.printAttribute("accesskey", accesskey);
		out.printAttribute("class", getCssClass());
		out.printAttribute("dir", dir);
		out.printAttribute("lang", lang);
		out.printAttribute("style", style);
		out.printAttribute("title", HtmlSanitizer.sanitize(getTranslatedTitle()));
	}
	
	/**
	 * @return the link title, translated if translateTitle = true 
	 */
	public String getTranslatedTitle() throws JspException {
		return translateTitle && title != null? (String)evaluateExpression(repository.guessString(title)) : title;
	}
	
	/**
	 * Same as prototype identify(), return current id or generate one if null
	 */
	public String identify() {
		if (id == null) {
			id = request.generateId("__id_");
		}
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String cls) {
		cssBuilder.setCss(cls);
	}
	
	/**
	 * Adds a new css class to this tag
	 */
	public void addCssClass(String cssClass) {
		cssBuilder.add(cssClass);
	}

	public void setStyle(String style) {
		this.style = style;
	}

	public String getCssClass() {
		return cssBuilder.getCss();
	}

	public String getStyle() {
		return style;
	}

	public void setAccesskey(String accesskey) {
		this.accesskey = accesskey;
	}

	public void setDir(String dir) {
		this.dir = dir;
	}

	public void setLang(String lang) {
		this.lang = lang;
	}

	/** 
	 * HTML attribute with different meanings:
	 * <ul>
	 * <li><a href="http://www.w3.org/TR/html4/interact/forms.html#adef-name-FORM">form</a></li>
	 * <li><a href="http://www.w3.org/TR/html4/struct/objects.html#adef-name-IMG">img</a></li>
	 * <li><a href="http://www.w3.org/TR/html4/struct/links.html#adef-name-A">url</a></li>
	 * </ul>
	 */
	@Attribute
	public void setName(String name) {
		this.name = name;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getId() {
		return id;
	}

	public String getAccesskey() {
		return accesskey;
	}

	public String getDir() {
		return dir;
	}

	public String getLang() {
		return lang;
	}

	public String getName() {
		return name;
	}

	public String getTitle() {
		return title;
	}

	public boolean isCssProcessed() {
		return cssProcessed;
	}

	public void setCssProcessed(boolean cssProcessed) {
		this.cssProcessed = cssProcessed;
	}

	public boolean isTranslateTitle() {
		return translateTitle;
	}

	public void setTranslateTitle(boolean translateTitle) {
		this.translateTitle = translateTitle;
	}

	public CssBuilder getCssBuilder() {
		return cssBuilder;
	}

	public boolean isIf() {
		return ifClause;
	}

	/**
	 * Set to false to skip tag execution.
	 */
	@Attribute
	public void setIf(boolean ifClause) {
		this.ifClause = ifClause;
	}
	
}
