package org.loom.tags;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import org.loom.config.Config;
import org.loom.i18n.MessagesRepository;
import org.loom.mapping.ParsedAction;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletRequestImpl;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.LoomServletResponseImpl;
import org.loom.servlet.Scope;
import org.loom.url.ParameterContainer;

/**
 * Parent of any Loom tag
 * @author icoloma
 *
 */
public abstract class AbstractTag extends SimpleTagSupport implements LoomTag, DynamicAttributes {

	/** the request instance that is being served */
	protected LoomServletRequest request;
	
	/** the response instance that is being written */
	protected LoomServletResponse response;
	
	/** the MessagesRepository instance */
	protected MessagesRepository repository;
	
	/** the WriterWrapper instance */
	protected WriterWrapper<?> out;
	
	/** when buffering the tag body, the buffered output */
	private StringWriterWrapper bufferedBody;
	
	/** extra attributes */
	private TagAttributes extendedAttributes = new TagAttributes();
	
	/**
	 * Initializes the tag attributes
	 */
	protected void initTag() {
		setRequest(LoomServletRequestImpl.getInstance(getPageContext().getRequest()));
		setResponse(LoomServletResponseImpl.getInstance(getPageContext().getResponse()));
		out = createWriterWrapper();
	}
	
	/**
	 * Find a containing ancestor with the required class.
	 * If not found, throws {@link IllegalArgumentException}
	 */
	@SuppressWarnings("unchecked")
	protected <T> T getAncestorWithClass(Class<T> clazz) {
		T container = (T) SimpleTagSupport.findAncestorWithClass(this, clazz);
		if (container == null) {
			throw new IllegalArgumentException(getClass().getSimpleName() + " can only be used nested inside a " + clazz.getSimpleName());
		}
		return container;
	}
	
	/**
	 * Does the tag cleanup, if any. 
	 */
	protected void endTag() {
	}
	
	/**
	 * Initializes the Request and PageData properties, and forward the call to the child class
	 */
	@Override
	public void doTag() throws JspException, IOException {
		
		initTag();
		try {
			doTagImpl();
		} finally {
			endTag();
		}
	}
	
	/**
	 * The method that must be implemented by the child class
	 */
	public abstract void doTagImpl() throws JspException, IOException;
	
	protected WriterWrapper<?> createWriterWrapper() {
		return new JspWriterWrapper(getPageContext().getOut());
	}
	
	public PageContext getPageContext() {
		return (PageContext) getJspContext();
	}
	
	/**
	 * Invokes a tag body using a StringBuffer, to delay its output
	 */
	protected String doBufferedBody() throws JspException, IOException {
		bufferedBody = new StringWriterWrapper();
		doBody(bufferedBody);
		return bufferedBody.getContents();
	}
	
	/**
	 * Invokes a tag body
	 * @return true if there was a body, false if not
	 */
	protected boolean doBody() throws JspException, IOException {
		return doBody(out);
	}
	
	private boolean doBody(WriterWrapper<?> out) throws JspException, IOException {
		JspFragment fragment = getJspBody();
		if (fragment != null) {
			fragment.invoke(out.getWriter());
			return true;
		}
		return false;
	}

	
	/**
	 * Create a scoped attribute
	 * @param name the attribute name
	 * @param value the value 
	 * @param scope the scope of the attribute. Cannot be null. 
	 */
	public void setScopedAttribute(String name, Object value, Scope scope) {
		if (scope == Scope.PAGE) {
			getPageContext().setAttribute(name, value, scope.getId());
		} else {
			request.setScopedAttribute(name, value, scope);
		}
	}
	
	/**
	 * Remove a scoped attribute
	 * @param name the var name
	 * @param scope the scope of the attribute. Cannot be null. 
	 */
	public void removeScopedAttribute(String name, Scope scope) {
		if (scope == Scope.PAGE) {
			getPageContext().removeAttribute(name);
		} else {
			request.removeScopedAttribute(name, scope);
		}
	}
	
	/**
	 * Get a scoped var
	 * @param name the var name
	 * @param scope the scope of the var. If null checks them all
	 */
	public Object getScopedAttribute(String name, Scope scope) {
		if (scope == null  || Scope.PAGE == scope) {
			Object value = getPageContext().getAttribute(name);
			if (Scope.PAGE == scope || value != null) {
				return value;
			}
		} 
		return scope == null? request.getScopedAttribute(name) : request.getScopedAttribute(name, scope);
	}
	
	/**
	 * @return the value of an action property or null if there is no parsed action
	 */
	protected Object getActionProperty(String propertyName) {
		ParsedAction action = request.getParsedAction();
		if (action != null) {
			return action.getPropertyAsObject(propertyName); 
		}
		return null;
	}
	
	/**
	 * Add an attribute not suported by the HTML 5 spec
	 */
	public void setExtendedAttribute(String name, Object value) {
		extendedAttributes.set(name, value);
	}
	
	/**
	 * @param expression the expression to be evaluated, such as ${something} + ${orTheOther}
	 * @return the evaluated response
	 */
	public Object evaluateExpression(String expression) throws JspException {
		if (expression == null || expression.indexOf('$') == -1) {
			return expression;
		}
		return Config.getInstance().getElProxy().evaluateExpression(getPageContext(), expression);
	}

	/**
	 * Print the extended attributes
	 */
	protected void printParentAttributes() throws IOException, JspException {
		extendedAttributes.print(out);
	}

	public void setRequest(LoomServletRequest request) {
		this.request = request;
		this.repository = request.getMessagesRepository();
	}
	
	/**
	 * Admit extra attributes like:
	 * <ul><li>data-* attributes: These will get appended as extendedAttributes, for further use by 
	 * javascript code</li>
	 * <li>param-* attributes: If the tag implements ParameterContainer, these attributes can be used 
	 * as a replacement for l:param</li>
	 * </ul>
	 */
    public void setDynamicAttribute( String uri, String localName, Object value ) throws JspException {
    	String extendedPrefix = extendedAttributes.getPrefix();
    	final String paramPrefix = "param-";
		if (localName.startsWith(extendedPrefix)) {
    		setExtendedAttribute(localName.substring(extendedPrefix.length()), value);
    	} else if (localName.startsWith(paramPrefix) && this instanceof ParameterContainer) {
    		((ParameterContainer)this).addParameter(localName.substring(paramPrefix.length()), value);
    	} else {
    		throw new JspException("Tag " + getClass().getName() + " does not have any attribute with name " + localName);
    	}
    }

	public LoomServletRequest getRequest() {
		return request;
	}
	
	public LoomServletResponse getResponse() {
		return response;
	}

	public WriterWrapper<?> getWriter() {
		return out;
	}

	public void setWriter(WriterWrapper<?> out) {
		this.out = out;
	}

	public MessagesRepository getMessagesRepository() {
		return repository;
	}

	public TagAttributes getExtendedAttributes() {
		return extendedAttributes;
	}

	public void setResponse(LoomServletResponse response) {
		this.response = response;
	}

	protected StringWriterWrapper getBufferedBody() {
		return bufferedBody;
	}
}
