/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.beans.SimpleBeanInfo;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.loom.log.Log;
import org.loom.util.ClassUtils;

/**
 * The BeanInfo standard class is used by the JSP compiler to bind setters and getters
 * to the tag attributes defined in the tld.
 * JSP tags do not allow setClass(String) because it messes up with getClass(). This BeanInfo
 * class fixes this.
 * To use, just create a BeanInfo class for the intended tag, and make it extend this class. 
 * It will create a "class" property mapped to setCssClass and getCssClass
 * 
 * @author Ignacio Coloma
 */
public abstract class CssClassBeanInfoSupport extends SimpleBeanInfo {
	
	private PropertyDescriptor[] propertyDescriptors;

	private Class<?> clazz;
	
	private static Log log = Log.getLog();
	
	public CssClassBeanInfoSupport() {
		String tagClassName = StringUtils.substringBeforeLast(getClass().getName(), "BeanInfo");
		this.clazz = ClassUtils.forName(tagClassName);
	}

	public PropertyDescriptor[] getPropertyDescriptors() {
		if (propertyDescriptors == null) {
			
			try {
				
				List<PropertyDescriptor> descriptorsAsList = new ArrayList<PropertyDescriptor>();
				Method setter = clazz.getMethod("setCssClass", String.class);
				descriptorsAsList.add(new PropertyDescriptor("class", null, setter));

				// Now add all the vanilla properties
				Method[] methods = clazz.getMethods();
				for (int i = 0; i < methods.length; i++) {
					Method m = methods[i];
					String methodName = m.getName();
					if (!Modifier.isStatic(m.getModifiers()) && methodName.startsWith("set") && !methodName.equals("setCssClass") && 
							m.getParameterTypes().length == 1) {
						descriptorsAsList.add(new PropertyDescriptor(getPropertyName(methodName), null, m));
					}
				}
				propertyDescriptors = descriptorsAsList.toArray(new PropertyDescriptor[descriptorsAsList.size()]);
				
			} catch (NoSuchMethodException e) {
				log.error(e, "Could not instantiate BeanInfo for class ", clazz.getName(), ": ", e);
				throw new RuntimeException(e);
			} catch (IntrospectionException e) {
				log.error(e, "Could not instantiate BeanInfo for class ", clazz.getName(), ": ", e);
				throw new RuntimeException(e);
			}

		}
		return propertyDescriptors;
	}

	/**
	 * @return the property name corresponding to a public setter name
	 */
	protected String getPropertyName(String setterName) {
		return Character.toLowerCase(setterName.charAt(3)) + setterName.substring(4);
	}
}
