/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.config.ConfigProperties;
import org.loom.converter.Converter;
import org.loom.i18n.MessagesRepository;
import org.loom.tags.WriterWrapper;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;
import org.loom.util.HtmlSanitizer;

/**
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#edef-SELECT">HTML select tag</a>
 * @author icoloma
 *
 */
@Tag(
	bodyContent=BodyContent.SCRIPTLESS,
	example=
		"&lt;l:inputSelect name=\"mortgage.country\" options=\"${action.countries}\" />\n" + 
		"&lt;l:inputSelect name=\"customer.type\">\n" +
		"	&lt;l:option label=\"--\" value=\"\" translateLabel=\"false\"/>\n" + 
		"	&lt;l:option value=\"NORMAL\" />\n" + 
		"	&lt;l:option value=\"BULK\" />\n" + 
		"&lt;/l:inputSelect>" 
)
public class InputSelectTag extends AbstractOptionsTag {
	
	private static String DEFAULT_CSS = ConfigProperties.getInstance().getDefaultCss(InputSelectTag.class, null);

	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-size-SELECT">HTML attribute</a> */
	@Attribute
	private Integer size;

	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-multiple">HTML attribute</a> */
	@Attribute
	private Boolean multiple;
	
	public InputSelectTag() {
		setCssClass(DEFAULT_CSS);
	}
	
	/**
	 * Prints this instance attributes
	 */
	@Override
	public void doTagImpl() throws JspException, IOException {
		calculateOptions();
		out.print("<select");
		printParentAttributes();
		out.printAttribute("multiple", multiple);
		out.printAttribute("size", size);
		out.print(">\n");
		writeOptions(out);
		out.print("</select>");

	}

	protected void writeOptions(WriterWrapper<?> out) throws IOException {
		Object currentValue = getValueAsObject();
		Converter converter = getConverter();
		MessagesRepository messagesRepository = getMessagesRepository();
		for (Option option : getNestedOptions()) {
			out.print("<option");
			String valueAsText = HtmlSanitizer.sanitize(converter.getAsText(option.getValue(), messagesRepository));
			out.printAttribute("value", valueAsText == null? "" : valueAsText);
			if (option.isSelected(currentValue)) {
				out.printAttribute("selected", "true");
			}
			out.print(">");
			out.print(option.getLabel(repository));
			out.print("</option>\n");
		}
	}

	public void setMultiple(Boolean multiple) {
		this.multiple = multiple;
	}

	public void setSize(Integer size) {
		this.size = size;
	}

}
