/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.exception;

import java.io.IOException;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.joda.time.LocalDate;
import org.loom.exception.LocaleAwareThrowable;
import org.loom.i18n.Message;
import org.loom.i18n.MessagesRepository;
import org.loom.i18n.MessagesRepositoryToMapAdapter;
import org.loom.log.Log;
import org.loom.servlet.names.RequestAttributeNames;
import org.loom.tags.WriterWrapper;
import org.loom.util.ClassUtils;
import org.loom.util.ExceptionUtils;
import org.loom.util.HtmlSanitizer;

import com.google.common.collect.Sets;

/**
 * If we are handling an http error, render the message and an explanation.
 * If we are handling a java exception:
 * <ul><li>If the exception implements LocaleAwareException and there is a MessagesRepository instance, 
 * render the translated message</li>
 * <li>If the exception is in the list of rendered exceptions, render its message</li>
 * <li>Else, render the default error message</li>
 * </ul>
 * 
 * @author icoloma
 */
@SuppressWarnings("unchecked")
public class DefaultErrorRenderer implements ErrorRenderer {

	/** true to log the request contents (headers, params, attributes) when logging an exception, default false */
	private boolean logRequest;
	
	/** true to clean the stack trace removing unneeded exception wrappers. Defaults to true */
	private boolean cleanStackTrace = true;

	/** the tag used to render the error message (exception message or http error message), empty or null to skip error message, default h1 */
	private String titleTag = "h1";
	
	/** the tag used to render the error explanation, empty or null to skip error detail, default div */
	private String messageTag = "div";
	
	private static Log log = Log.getLog();
	
	/** the list of exceptions that will not be hidden from the user */
	private static Set<Class<? extends Throwable>> renderedExceptions = Sets.newHashSet();

	static {
		String className = "org.springframework.security.SpringSecurityException";
		if (ClassUtils.isPresent(className)) {
			addRenderedException((Class<? extends Throwable>) ClassUtils.forName(className));
		}
	}
	
	/**
	 * Add a exception that should be rendered on the web page
	 * @param exception
	 */
	public static void addRenderedException(Class<? extends Throwable> exception) {
		renderedExceptions.add(exception);
	}
	
	public void renderError(WriterWrapper<?> out, HttpServletRequest request) throws IOException {
		Throwable exception = extractException(request);
		if (exception == null) {
			renderHttpError(out, request);
		} else {
			renderException(out, request, exception);
		}
	}
	
	/**
	 * Render a tag (message or detail)
	 */
	private void renderTag(WriterWrapper<?> out, String tagName, String id, String css, String text) throws IOException {
		if (!StringUtils.isEmpty(tagName)) {
			out.print("<").print(tagName);
			out.printAttribute("id", id);
			out.printAttribute("class", css);
			out.print(">");
			out.print(text == null? "" : HtmlSanitizer.sanitize(text));
			out.print("</").print(tagName).print(">\n");
		}
	}
	
	/**
	 * Render an http error message
	 */
	private void renderHttpError(WriterWrapper<?> out, HttpServletRequest request) throws IOException {
		Integer httpStatus = (Integer) request.getAttribute(RequestAttributeNames.HTTP_ERROR);
		if (httpStatus == null) {
			log.error("HTTP status code is null. This should not happen, request contents follow.\n", request);
		}
		
		// this should not be null for a http error if the filter is properly configured
		MessagesRepository messagesRepository = getMessagesRepository(request);
		renderTag(out, titleTag, "message", "http http-" + httpStatus, messagesRepository.getRequiredString("http." + httpStatus));
		renderTag(out, messageTag, "detail", "http http-" + httpStatus, messagesRepository.getRequiredString("http." + httpStatus + ".verbose"));
	}

	/**
	 * Render a java exception
	 */
	private void renderException(WriterWrapper<?> out, HttpServletRequest request, Throwable exception) throws IOException {
		
		MessagesRepository messagesRepository = getMessagesRepository(request);
		String title = messagesRepository == null? "Your request could not be processed" : messagesRepository.getRequiredString("http.exceptionTitle");
		String exceptionMessage = getExceptionMessage(request, exception);
		renderTag(out, titleTag, "message", "exception", title);
		renderTag(out, messageTag, "detail", "exception", exceptionMessage);
		log.debug(exception, exceptionMessage, logRequest? "" : "\n" + request);
	}
	
	/**
	 * @return the exception message, translated if possible
	 */
	protected String getExceptionMessage(HttpServletRequest request, Throwable exception) {
		// if the message is locale-aware
		MessagesRepository messagesRepository = getMessagesRepository(request);
		if (exception instanceof LocaleAwareThrowable && messagesRepository != null) {
			Message message = ((LocaleAwareThrowable)exception).getLocaleAwareMessage();
			return messagesRepository.translateMessage(message);
		} else if (renderExceptionMessage(exception)) {
			return exception.getMessage();
		} else {
			String issueNumber = createNewIssueNumber();
			if (messagesRepository != null) {
				Message msg = new Message("http.exceptionMessage").addArg("issueNumber", issueNumber);
				return messagesRepository.translateMessage(msg);
			} else {
				return "Please take note of your issue number #" + issueNumber + " and contact technical support."; 
			}
		}

	}
	
	/**
	 * Return true if the exception message is in the list of rendered messages.  Normal is false.
	 * @return true if the exception message should be rendered.
	 */
	private boolean renderExceptionMessage(Throwable exception) {
		if (exception instanceof LocaleAwareThrowable) {
			return true;
		}
		Class<? extends Throwable> exceptionClass = exception.getClass();
		for (Class<? extends Throwable> clazz : renderedExceptions) {
			if (clazz.isAssignableFrom(exceptionClass)) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Override this method to change the format of the issue number.
	 */
	protected String createNewIssueNumber() {
		return new LocalDate().toString("yyMMdd") + System.currentTimeMillis() % 10000;
	}
	
	/**
	 * Return the messagesRepository instance.
	 * Note that this renderer is an error handler, so you should not suppose that 
	 * there is a valid MessagesRepository. This method may return null if no instance is found.
	 */
	protected MessagesRepository getMessagesRepository(HttpServletRequest request) {
		MessagesRepositoryToMapAdapter adapter = (MessagesRepositoryToMapAdapter) request.getAttribute(RequestAttributeNames.MESSAGES_REPOSITORY);
		return adapter == null? null : (MessagesRepository) adapter.getMessagesRepository();
	}

	/**
	 * Extract the exception from the request object
	 */
	private Throwable extractException(HttpServletRequest request) {
		Throwable exception = (Throwable) request.getAttribute(RequestAttributeNames.JAVA_EXCEPTION);
		if (exception == null) {
			// spring security does this for a 403
			exception = (Throwable) request.getAttribute(RequestAttributeNames.SPRING_SECURITY_403_EXCEPTION);
		}
		if (exception != null && cleanStackTrace) {
			exception = ExceptionUtils.getNestedException(exception);
		}
		return exception;

	}
	
	public void setLogRequest(boolean logRequest) {
		this.logRequest = logRequest;
	}

	public void setCleanStackTrace(boolean cleanStackTrace) {
		this.cleanStackTrace = cleanStackTrace;
	}

	public void setTitleTag(String messageTag) {
		this.titleTag = messageTag;
	}

	public void setMessageTag(String messageTag) {
		this.messageTag = messageTag;
	}

}
