/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.menu;

import java.util.ArrayList;
import java.util.List;

import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.tags.core.UrlTag;
import org.loom.url.UrlBuilder;

/**
 * One menu item.
 * @author icoloma
 */
public class MenuItemData implements MenuItemDataCollection {
	
	/** the parent path, MenuData or MenuItem if this is level 0 menu item */
	private MenuItemDataCollection parent;
	
	/** the css class specified in the menu tag */
	private String cssClass;
	
	/** the html id of this node */
	private String id;
	
	/** the text to display */
	private String label;
	
	/** true if the label should be translated (translated labels with EL expressions will be executed anyway) */
	private boolean translateLabel = true;
	
	/** the link 'target' attribute, to open in a different window */
	private String target;
	
	/** the target action, event and params of this menu item. May be null (if there is no target url
	 * and this menuItem only holds a submenu which will be displayed using javascript) */
	private UrlBuilder urlBuilder;
	
	/** a list of events that, when being called, imply that this menu item is  marked as "selected" even when the configured event is other (null if none) */
	private String[] aliases;

	/** the list of children of this menu object, null if none */
	private List<MenuItemData> children;
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * @return a UrlTag that can render the link for this menu item
	 */
	public UrlTag getUrlTag(LoomServletRequest request, LoomServletResponse response) {
		UrlTag url = new UrlTag();
		url.setRequest(request);
		url.setResponse(response);
		url.setTarget(target);
		url.setTitle(label);
		url.setTranslateTitle(translateLabel);
		if (urlBuilder != null && !urlBuilder.isEmpty()) {
			url.setUrl(urlBuilder);
			if (urlBuilder.hasEvent()) {
				urlBuilder.getEvent().decorate(url);
			}
		} else {
			url.setHref("#");
		}
		return url;
	}

	/**
	 * Return true if this instance is an ancestor of the parameter
	 */
	public boolean isAncestorOf(MenuItemData item) {
		if (item == null) {
			return false;
		}
		for (; item != null; item = (MenuItemData)item.getParent()) {
			if (item == this) {
				return true;
			}
		}
		return false;
	}

	public List<MenuItemData> getChildren() {
		return children;
	}
	
	public boolean hasChildren() {
		return children != null;
	}
	
	public void addChild(MenuItemData item) {
		if (children == null) {
			children = new ArrayList<MenuItemData>();
		}
		children.add(item);
		item.setParent(this);
	}

	public String getCssClass() {
		return cssClass;
	}

	public void setCssClass(String cssClass) {
		this.cssClass = cssClass;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public void setParent(MenuItemDataCollection parent) {
		this.parent = parent;
	}
	
	public String getLabel() {
		if (label == null && urlBuilder.getActionMapping() != null) {
			return urlBuilder.calculateTitle();
		}
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public UrlBuilder getUrlBuilder() {
		return urlBuilder;
	}

	public void setUrlBuilder(UrlBuilder urlBuilder) {
		this.urlBuilder = urlBuilder;
	}

	public boolean isTranslateLabel() {
		return translateLabel;
	}

	public void setTranslateLabel(boolean i18n) {
		this.translateLabel = i18n;
	}

	public String[] getAliases() {
		return aliases;
	}

	public void setAliases(String[] aliases) {
		this.aliases = aliases;
	}

	public String getTarget() {
		return target;
	}

	public void setTarget(String target) {
		this.target = target;
	}

	/**
	 * @return true if this menuItem does not have specified either href or action that should be invoked when clicked
	 */
	public boolean isEmpty() {
		return urlBuilder == null || urlBuilder.isEmpty();
	}
	
	public void addParameter(String name, Object value) {
		getUrlBuilder().add(name, value);
	}
	
	public MenuItemDataCollection getParent() {
		return parent;
	}
	
}
