/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.menu;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.apache.commons.lang.StringUtils;
import org.loom.tags.AbstractTag;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Menu item.
 * Menu items will detect if the current url is the same they point to, and set their CSS
 * accordingly. Forbidden URLs will also get a "unauthorized" CSS class automatically added.
 * 
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.SCRIPTLESS,
		example="See <a href=\"menu.html\">menu</a>"
	)
public class MenuItemTag extends AbstractTag implements ParameterContainer, MenuItemTagCollection {
	
	/** contains the destination url of this menu item */
	private UrlBuilder urlBuilder = new UrlBuilder();
	
	/** the data of this menuItem */
	private MenuItemData item = new MenuItemData();
	
	private boolean ifClause = true;
	
	@Override
	public void doTagImpl() throws IOException, JspException {
		if (!ifClause) {
			return;
		}
		MenuItemTagCollection parent = getAncestorWithClass(MenuItemTagCollection.class);
		doBody();
		item.setUrlBuilder(urlBuilder);
		parent.addChild(item);
	}

	/**
	 * Sets the text to be shown in this menu item
	 */
	@Attribute
	public void setLabel(String label) {
		item.setLabel(label);
	}
	
	/**
	 * Set to true to translate the item text, default true
	 */
	@Attribute
	public void setTranslateLabel(boolean translateLabel) {
		item.setTranslateLabel(translateLabel);
	}
	
	/**
	 * Set to true to translate the item text, default true
	 * @deprecated use translateLabel instead
	 */
	@Attribute
	public void setI18n(boolean i18n) {
		item.setTranslateLabel(i18n);
	}
	
	/** 
	 * A list of events that will mark this menu item as "selected",
	 * even if the target event is different 
	 */
	@Attribute
	public void setAliases(String a) {
		String []aliases = StringUtils.split(a, ", ");
		item.setAliases(aliases);
	}
	
	public MenuItemTag addParameter(String name, Object value) {
		urlBuilder.add(name, value);
		return this;
	}
	
	/**
	 * The action this link points to. Either this or href must be specified.
	 */
	@Attribute
	public void setAction(String action) {
		urlBuilder.setAction(action);
	}
	
	/**
	 * The event this link points to. If action is specified but event is not, the
	 * default event will be assumed. If the action does not define a default event,
	 * an exception will be thrown.
	 * Either action/event or href must be set.
	 */
	@Attribute
	public void setEvent(String event) {
		urlBuilder.setEvent(event);
	}
	
	/**
	 * The mapping as <code>action.event</code>. This method is equivalent to 
	 * setting <code>action</code> and <code>event</code> separately.
	 */
	@Attribute
	public void setMapping(String mapping) {
		this.urlBuilder.setMapping(mapping);
	}
	
	/** 
	 * The target url. Either this or the action / event pair must be set.
	 */
	@Attribute
	public void setHref(String url) {
		urlBuilder.setRawUrl(url);
	}
	
	/**
	 * The hash (the part after '#') of the url 
	 */
	@Attribute
	public void setHash(String hash) {
		urlBuilder.setHash(hash);
	}
	
	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String cls) {
		item.setCssClass(cls);
	}

	public void addChild(MenuItemData child) {
		this.item.addChild(child);
	}
	
	/** <a href="http://www.w3.org/TR/html4/struct/global.html#adef-id">HTML attribute</a> */
	@Attribute
	public void setId(String id) {
		item.setId(id);
	}

	/** <a href="http://www.w3.org/TR/html4/present/frames.html#adef-target">HTML attribute</a> */
	@Attribute
	public void setTarget(String target) {
		this.item.setTarget(target);
	}
	
	/**
	 * Set to false to skip tag execution.
	 */
	@Attribute
	public void setIf(boolean ifClause) {
		this.ifClause = ifClause;
	}

}
