/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.menu;

import static org.loom.servlet.names.CssClassNames.UNAUTHORIZED;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.tags.CssBuilder;
import org.loom.tags.StringWriterWrapper;
import org.loom.tags.WriterWrapper;
import org.loom.tags.core.UrlTag;

/**
 * Decorates menu items as a list (ul) of nested menu items and submenus.
 * @author icoloma
 */
public class NestedMenuDecorator extends AbstractMenuDecorator {

	/** true to render empty menu items, default false */
	private boolean renderEmptyItems;
	
	public boolean beforeDoTag(MenuTag tag) throws IOException,
	JspException {
		return true;
	}
	
	public void afterDoTag(MenuTag tag) throws IOException, JspException {
		AbstractMenuData menu = tag.getMenu();
		WriterWrapper<?> out = tag.getWriter();
		out.print("<ul");
		out.printAttribute("id", menu.getId());
		out.printAttribute("class", menu.getCssClass());
		out.print(">\n");
		MenuItemData selectedItem = menu.getSelectedItem(tag.getRequest());
		for (MenuItemData item : menu.getChildren()) {
			out.print(renderMenuItem(tag, item, selectedItem));
		}
		out.print("</ul>");
	}

	/**
	 * Return the string that represents this menu item, empty if it should not be rendered
	 * @param item the current item being printed
	 * @param selectedItem the currently selected item in this menu
	 */
	public String renderMenuItem(MenuTag tag, MenuItemData item, MenuItemData selectedItem) throws IOException, JspException {
		
		// if the 'unauthorized' CSS class is found, this menu item will not be rendered
		UrlTag urlTag = item.getUrlTag(tag.getRequest(), tag.getResponse());
		if (!isRenderUnauthorizedItems() && urlTag.getCssBuilder().contains(UNAUTHORIZED)) {
			return "";
		}

		// first, prepare its children contents
		String childrenContents = "";
		if (item.hasChildren()) {
			StringBuilder builder = new StringBuilder(item.getChildren().size() * 80);
			for (MenuItemData child : item.getChildren()) {
				builder.append(renderMenuItem(tag, child, selectedItem));
			}
			childrenContents = builder.toString();
		}
		if (childrenContents.length() == 0 && item.isEmpty() && !renderEmptyItems) {
			return "";
		}
		
		// Render the item and its children
		StringWriterWrapper out = new StringWriterWrapper();
		out.print("<li");
		out.printAttribute("id", item.getId());
		out.printAttribute("class", calculateCss(item, selectedItem));
		out.print(">");
		
		urlTag.setWriter(out);
		urlTag.setJspContext(tag.getPageContext());
		urlTag.doTagImpl();
		
		if (childrenContents.length() > 0) {
			out.print("\n<ul>\n").print(childrenContents).print("</ul>");
		}
		out.print("</li>\n");
		return out.getContents();
	}
	
	/**
	 * Calculate the css class of this menuItem.
	 * This method adds the following css classes:
	 * <ul><li><code>selected</code> if this is the currently selected item</li>
	 * <li><code>submenu</code> if this menu item has a nested submenu below</li>
	 * </ul>
	 * All menuItem classes are also expanded because of accessibility: in a world 
	 * without javascript, all menu items should be displayed expanded, so we 
	 * collapse the unselected items using javascript instead of the other way.
	 */
	public String calculateCss(MenuItemData item, MenuItemData selectedItem) {
		CssBuilder css = new CssBuilder();
		css.add(item.getCssClass());
		css.add("expanded");
		if (selectedItem != null) {
			if (item == selectedItem) {
				css.add("selected");
			} 
		}
		if (item.hasChildren()) {
			css.add("submenu");
		}
		
		return css.getCss();
	}

	public boolean isRenderEmptyItems() {
		return renderEmptyItems;
	}

	public void setRenderEmptyItems(boolean renderEmptyItems) {
		this.renderEmptyItems = renderEmptyItems;
	}

}
