package org.loom.tags.messages;

import org.apache.commons.lang.StringUtils;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Format a lapse of time in seconds/minutes/hours/days.  
 * Equivalent to <a href="http://api.rubyonrails.org/classes/ActionView/Helpers/DateHelper.html#M001695">DateHelper.distance_of_time_in_words</a> in Ruby
 * and Date.toDuration().humanized() in format.js
 * @author icoloma
 *
 */
@Tag(
		bodyContent=BodyContent.SCRIPTLESS,
		example="&lt;l:formatDuration value=\"${myDurationInMillis}\"/>"
	)
public class FormatDurationTag extends AbstractFormatTag {
	
	/** durations per unit, in seconds */
	private static int unitDurations[] = new int[] { 
		365 * 24 * 60 * 60, // years 
		30 * 24 * 60 * 60,  // months
		7 * 24 * 60 * 60,   // weeks
		24 * 60 * 60,       // days
		60 * 60,            // hours
		60,                 // minutes
		1					// seconds
	};
	
	/** interval to format, in milliseconds */
	@Attribute(required=true)
	private long value;

	/** the number of maximum  units to use (years, months, days, etc). Default is 2 */
	@Attribute
	private int maxUnits = 2;
	
	@Override
	public String format() {
		String[] unitNames = StringUtils.split(repository.getString("loom.format.timeUnitNames"), ' ');
		String[] singularUnitNames = StringUtils.split(repository.getString("loom.format.singularTimeUnitNames"), ' ');
		StringBuilder sb = new StringBuilder();
		long v = value / 1000;
		int unitsCount = 0;
		for (int i = 0; i < unitDurations.length && unitsCount < maxUnits; i++) {
			int duration = unitDurations[i];
			int unitAmount = (int)(v / duration);
			if (unitAmount > 0 || (sb.length() == 0 && i == unitDurations.length - 1)) {
				if (sb.length() > 0) {
					sb.append(" ");
				}
				String unitName = unitAmount == 1? singularUnitNames[i] : unitNames[i];
				sb.append(unitAmount).append(" ").append(unitName);
				unitsCount++;
			}
			v -= unitAmount * duration;
		}
		return sb.toString();
	}

	public void setValue(long value) {
		this.value = value;
	}

	public void setMaxUnits(int maxUnitsCount) {
		this.maxUnits = maxUnitsCount;
	}

}
