package org.loom.tags.paged;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import org.loom.i18n.Message;
import org.loom.mapping.ParsedAction;
import org.loom.paged.PagedListData;
import org.loom.paged.PagedListDataImpl;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.tags.AbstractTag;
import org.loom.tags.CssBuilder;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;

import com.google.common.collect.Lists;

/**
 * Parent of paged containers (tables or lists)
 * @author icoloma
 *
 */
public abstract class AbstractPagedContainerTag  extends AbstractTag implements ParameterContainer {

	/** the position where the links bar will be displayed */
	private enum Position {
		TOP, BOTTOM, BOTH, NONE
	}
	
	/** the action/event that handles any column reordering */
	private UrlBuilder url = new UrlBuilder();
	
	/** the name of the request attribute that will contain the current row value, default "row" */
	@Attribute
	private String rowName = "row";
	
	/** the name of the request attribute that will contain the current row index, default "index" */
	@Attribute
	private String indexName = "index";
	
	/** 
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-id">HTML attribute</a>.
	 */
	@Attribute
	private String id;
	
	/** the css class of this table, defaults to paged-list */
	private String cssClass = "paged";
	
	/** <a href="http://www.w3.org/TR/html4/present/styles.html#adef-style">HTML attribute</a> */
	@Attribute
	private String style;

	/** the PagedListData object. Either this or listData should be set */
	@Attribute
	private PagedListData<?> data;
	
	/** where to put the link bar */
	private Position linksPosition = Position.TOP;
	
	/** where to put the overview message */
	private Position overviewPosition = Position.TOP;
	
	/** the number of page links to display, default 10 */
	@Attribute
	private int linksCount = 10;
	
	/** the index of the current row, starts at 0 */
	private int rowIndex;

	/** the current row object */
	private Object rowObject;
	
	/** the HTML class of each generated TR. If set, any @{...} expression will be executed by the EL engine for each row */
	@Attribute
	private String rowClass;
	
	/** true to generate even/odd css classes for each row, default true */
	@Attribute
	private boolean alternateRows = true;
	
	/** the message prefix to be used, default "paged." */
	@Attribute
	private String messagePrefix = "paged.";
	
	/** false to render nothing if the list is empty. Default true */
	@Attribute
	private boolean renderIfEmpty = true;
	
	/** 
	 * true to append all GET parameters included in the current request when 
	 * building the table links, default true 
	 */
	@Attribute
	private boolean addUriParams = true;
	
	private boolean ifClause = true;

	@Override
	public void doTagImpl() throws JspException, IOException {
		
		if (!ifClause) {
			return;
		}
		
		if (!renderIfEmpty && (data == null || data.isEmpty())) {
			return;
		}
		
		if (data == null || data.getData() == null) { 
			data = PagedListDataImpl.EMPTY_LIST;
		} 
		
		if (data.getTotalPages() == -1) {
			throw new IllegalArgumentException("PagedListData.getTotalPages() has not been initialized.");
		}
		
		// if no action or event has been set, initialize to the same of current request
		ParsedAction parsedAction = request.getParsedAction();
		if (url.isEmpty() && parsedAction != null) {
			url.setEvent(parsedAction.getEvent());
		}
		url.setIncludeRequestParameters(addUriParams);
		
		// add current ordenation parameters to the target url
		String sp = withPrefix(RequestParameterNames.PAGED_SORT_PROPERTY);
		String pso = withPrefix(RequestParameterNames.PAGED_SORT_ORDER);
		url.replace(sp, request.getParameter(sp));
		url.replace(pso, request.getParameter(pso));

		out.start("div", "class", cssClass, "style", style);
		
		renderOverview(Position.TOP);
		renderLinks(Position.TOP);
		
		if (!data.isEmpty()) {
			renderContainer();
		}
		
		renderLinks(Position.BOTTOM);
		renderOverview(Position.BOTTOM);
		
		out.print("</div>\n");
	}
	
	/**
	 * if id != null, return the provided parameter name prefixed by ${id.}
	 * else return the param name 
	 */
	public String withPrefix(String paramName) {
		return data.getId() == null? paramName : paramName + "-" + data.getId();
	}
	
	/**
	 * Render the table or list with the data
	 */
	protected void renderContainer() throws IOException, JspException {
		if (data.isEmpty()) {
			renderEmptyContainer();
		} else {
			renderHeader();
			rowIndex = 0;
			PageContext pc = getPageContext();
			for (Iterator<?> i = data.getData().iterator(); i.hasNext(); ) {
				rowObject = i.next();
				pc.setAttribute(rowName, rowObject);
				pc.setAttribute(indexName, rowIndex);
				renderRowStart();
				doBody();
				renderRowEnd();
				rowIndex++;
			}
			pc.removeAttribute(rowName);
			pc.removeAttribute(indexName);
			renderFooter();
		}
	}

	/**
	 * Render the header of the container component
	 */
	protected abstract void renderHeader() throws IOException, JspException;
	
	/**
	 * Render the footer of the container component
	 */
	protected abstract void renderFooter() throws IOException, JspException;
	
	/**
	 * Render the start of a new row
	 */
	protected abstract void renderRowStart() throws IOException, JspException;
	
	/**
	 * Render the end of a row
	 */
	protected abstract void renderRowEnd() throws IOException, JspException;

	/**
	 * Render an empty container
	 */
	protected abstract void renderEmptyContainer() throws IOException, JspException;
	
	/**
	 * Render the overview message
	 * @param position the position where we are rendering. It will only print the message if 
	 * overviewPosition is the same as the provided one, or it is set to BOTH
	 */
	public void renderOverview(Position position) throws IOException, JspException {
		if (overviewPosition != Position.BOTH && overviewPosition != position) {
			return;
		}
		out.print("<div").printAttribute("class", "overview").print(">");
		String messageKey = data.isEmpty()? "empty" : 
				data.getTotalRecords() == 1? "oneItem" : 
				data.getPageSize() >= data.getTotalRecords()? "allItems" :
				"partial";
		Message message = new Message(messagePrefix + messageKey);
		message.addArg("data", data);
		out.print(repository.translateMessage(message));
		out.print("</div>\n");
	}
	
	/**
	 * Render the next/previous links
	 * @param position the position where we  are rendering. It will only print the links if 
	 * linkPosition is the same as the provided one, or it is set to BOTH
	 */
	public void renderLinks(Position position) throws IOException, JspException {
		String ellipsis = "<li>&hellip;</li>";
		if (data.getPageSize() >= data.getTotalRecords() || linksPosition != Position.BOTH && linksPosition != position) {
			return;
		}
		
		// start the list of links
		out.print("<ul").printAttribute("class", "links").print(">\n");
		UrlBuilder url = this.url.clone();

		// 'previous' button and optional ellipsis
		printLink(url, repository.getRequiredString(messagePrefix + "previous"), "previous", data.getPageIndex() - 1);
		if (data.getPageIndex() > linksCount / 2) {
			out.print(ellipsis);
		}
		
		// links
		int index = Math.max(data.getPageIndex() - linksCount / 2, 0);
		int max = Math.min(index + linksCount, data.getTotalPages());
		while (index < max) {
			printLink(url, index++);
		}
		
		// 'next' button and optional ellipsis
		if (max < data.getTotalPages()) {
			out.print(ellipsis);
		}
		printLink(url, repository.getRequiredString(messagePrefix + "next"), "next", data.getPageIndex() + 1);
		out.print("</ul>\n");
	}
	
	protected void printLink(UrlBuilder url, int pageIndex) throws IOException {
		printLink(url, String.valueOf(pageIndex + 1), null, pageIndex);
	}
	
	/**
	 * Print a li item
	 * @param url the builder that is being reused to generate all links
	 * @param contents the contents of the tag
	 * @param pageIndex the pageIndex of the tag
	 */
	protected void printLink(UrlBuilder url, String contents, String cssClassName, int pageIndex) throws IOException {
		boolean inRange = isIndexInRange(pageIndex);
		CssBuilder css = new CssBuilder();
		css.add("link").add(cssClassName);
		if (pageIndex == data.getPageIndex()) {
			css.add("active"); 
		} else if (!inRange) {
			css.add("disabled"); 
		}
		
		out.print("<li").printAttribute("class", css.getCss()).print(">");
		String di = withPrefix(RequestParameterNames.PAGED_INDEX);
		if (inRange && pageIndex != data.getPageIndex()) {
			url.replace(di, pageIndex);
			out.print("<a").printAttribute("href", url.getURL()).print(">");
			out.print(contents);
			out.print("</a>");
		} else {
			out.print(contents);
		}
		out.print("</li> ");
	}
	
	/**
	 * @return true if the provided page index exists (if it's greater than zero and less than the total number of pages)
	 */
	public boolean isIndexInRange(int pageIndex) {
		return pageIndex >= 0 && pageIndex < data.getTotalPages();
	}
	
	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String css) {
		this.cssClass = css;
	}
	
	/**
	 * If set, this action will be used to generate the paged links. Default to the current action / event pair
	 */
	@Attribute
	public void setAction(String action) {
		url.setAction(action);
	}
	
	/**
	 * If set, this event will be used to generate the paged links. Default to the current action / event pair
	 */
	@Attribute
	public void setEvent(String event) {
		url.setEvent(event);
	}

	/**
	 * The mapping as <code>action.event</code>. This method is equivalent to 
	 * setting <code>action</code> and <code>event</code> separately.
	 */
	@Attribute
	public void setMapping(String mapping) {
		this.url.setMapping(mapping);
	}
	
	/**
	 * If set, this link will be used as the paged link. Default to the current action / event pair
	 */
	@Attribute
	public void setHref(String href) {
		url.setRawUrl(href);
	}
	
	public ParameterContainer addParameter(String name, Object value) {
		url.add(name, value);
		return this;
	}

	public String getRowName() {
		return rowName;
	}

	public void setRowName(String rowName) {
		this.rowName = rowName;
	}

	public String getRowClass() {
		try {
			// calculate any @{elexpression}
			String result = null;
			if (rowClass !=  null) {
				result = evaluateExpression(rowClass.replace("@{", "${")).toString();
			}
			if (alternateRows) {
				String alternateStyle = rowIndex % 2 == 0? "even" : "odd";
				return result == null? alternateStyle : result + " " + alternateStyle;
			} else {
				return result;
			}
		} catch (JspException e) {
			throw new RuntimeException(e);
		}
	}

	public void setRowClass(String rowCssClass) {
		this.rowClass = rowCssClass;
	}

	public UrlBuilder getUrl() {
		return url;
	}

	public String getId() {
		return id;
	}

	public PagedListData<?> getData() {
		return data;
	}

	public int getRowIndex() {
		return rowIndex;
	}
	public void setId(String id) {
		this.id = id;
	}

	public Object getRowObject() {
		return rowObject;
	}

	public void setData(PagedListData<?> data) {
		this.data = data;
	}
	
	/** 
	 * When set, this attribute will be converted to a single-paged data container. 
	 * This is a convenient method when displaying non-paginated collections. 
	 */
	@SuppressWarnings("unchecked")
	@Attribute
	public <T> void setCollection(Collection<T> data) {
		PagedListDataImpl<T> pdata = new PagedListDataImpl<T>();
		pdata.setPageIndex(0);
		pdata.setPageSize(data.size());
		pdata.setTotalRecords(data.size());
		pdata.setData(data instanceof List? (List<T>) data : Lists.newArrayList(data));
		this.data = pdata;
	}
	
	/** 
	 * Where to put the links bar. Possible values are 
	 * <code>top</code> (default), <code>bottom</code>, <code>both</code> and <code>none</code>
	 */
	@Attribute
	public void setLinksPosition(String position) {
		this.linksPosition = Position.valueOf(position.toUpperCase());
	}
	
	/** 
	 * Where to put the overview message. Possible values are 
	 * <code>top</code> (default), <code>bottom</code>, <code>both</code> and <code>none</code>
	 */
	@Attribute
	public void setOverviewPosition(String position) {
		this.overviewPosition = Position.valueOf(position.toUpperCase());
	}

	public void setLinksCount(int linksCount) {
		this.linksCount = linksCount;
	}

	public void setRowIndex(int rowIndex) {
		this.rowIndex = rowIndex;
	}

	public boolean isAlternateRows() {
		return alternateRows;
	}

	public void setAlternateRows(boolean differenciateRows) {
		this.alternateRows = differenciateRows;
	}
	
	/**
	 * If set, the table links will include this hash.
	 */
	@Attribute
	public void setHash(String hash) {
		this.url.setHash(hash);
	}

	public String getIndexName() {
		return indexName;
	}

	public void setIndexName(String indexName) {
		this.indexName = indexName;
	}

	public String getMessagePrefix() {
		return messagePrefix;
	}

	public void setMessagePrefix(String messagePrefix) {
		this.messagePrefix = messagePrefix.endsWith(".")? messagePrefix : messagePrefix + ".";
	}

	public void setRenderIfEmpty(boolean renderIfEmpty) {
		this.renderIfEmpty = renderIfEmpty;
	}

	public boolean isRenderIfEmpty() {
		return renderIfEmpty;
	}

	public void setStyle(String style) {
		this.style = style;
	}

	/**
	 * Set to false to skip tag execution.
	 */
	@Attribute
	public void setIf(boolean ifClause) {
		this.ifClause = ifClause;
	}

}
