/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.tabs;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.servlet.HttpMethod;
import org.loom.tags.AbstractDecoratedTag;
import org.loom.tags.CssBuilder;
import org.loom.tags.Link;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Tabs are rendered as a group of &lt;div> tags to be handled using javascript. 
 * When the browser displays the page the currently selected tab is extracted 
 * from the URL hash. Browsers with javascript disabled will see all the tabs at once.
 * <p>
 * For cases where a tab is too heavy to be included on each request (or pages with 
 * just too many tabs) the tabs can include an action/event pair. These tabs will 
 * only get rendered when the current request is the same pointed by the tab.
 * </p>
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.SCRIPTLESS,
		example=
			"&lt;l:tabContainer>\n" +
			"    &lt;l:tab id=\"tab1\">\n" +
			"            Tab1 contents\n" +
			"    &lt;/l:tab>\n" +
			"    &lt;l:tab id=\"tab2\" action=\"Dummy\" event=\"bar\" param-parameter1=\"baz\">\n" +
			"            Tab2 contents\n" +
			"    &lt;/l:tab>\n" +
			"&lt;/l:tabContainer>"
	)
public class TabTag extends AbstractDecoratedTag implements ParameterContainer, Link {

	/** the tab url target. Usually, this would have only the hash value set */
	private UrlBuilder url = new UrlBuilder();
	
	/** if not null, the tab link will open in another window */
	@Attribute
	private String target;
	
	private CssBuilder cssBuilder = new CssBuilder();
	
	/** the container of this tab */
	private TabContainerTag container;
	
	/** the HTTP method to use */
	private HttpMethod method;
	
	/** true to print the tab, default true */
	private boolean ifClause = true;
	
	@Override
	protected boolean renderTag() {
		return ifClause;
	}
	
	@Override
	protected void initTag() {
		super.initTag();
		if (container == null) {
			setContainer(getAncestorWithClass(TabContainerTag.class));
		}
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		// skip body if this tab has an action/event pair and it's not a match
		if (!url.hasEvent() || (request.getParsedAction() != null && request.getParsedAction().matches(url))) {
			doBody();
		}
	}

	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-id">HTML attribute</a>.
	 */
	@Attribute(required=true)
	public void setId(String id) {
		this.url.setHash(id);
	}
	
	public String getId() {
		return this.url.getHash();
	}
	
	/**
	 * If not null, this tab will point to the specified action/event pair and 
	 * clicking it will trigger a request to the server.  
	 */
	@Attribute
	public void setAction(String action) {
		this.url.setAction(action);
	}
	
	/**
	 * If not null, this tab will point to the specified action/event pair and 
	 * clicking it will trigger a request to the server.  
	 */
	@Attribute
	public void setEvent(String event) {
		this.url.setEvent(event);
	}

	/**
	 * The mapping as <code>action.event</code>. This method is equivalent to 
	 * setting <code>action</code> and <code>event</code> separately.
	 */
	@Attribute
	public void setMapping(String mapping) {
		this.url.setMapping(mapping);
	}
	
	public TabTag addParameter(String name, Object value) {
		this.url.add(name, value);
		return this;
	}

	public UrlBuilder getUrl() {
		return url;
	}

	public void setTarget(String target) {
		this.target = target;
	}

	public String getTarget() {
		return target;
	}
	
	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String cls) {
		cssBuilder.setCss(cls);
	}

	public void addCssClass(String css) {
		cssBuilder.add(css);
	}

	public CssBuilder getCssBuilder() {
		return cssBuilder;
	}

	public TabContainerTag getContainer() {
		return container;
	}

	public void setContainer(TabContainerTag container) {
		this.container = container;		
		container.addTab(this);
	}

	public HttpMethod getMethod() {
		return method;
	}

	public void setMethod(HttpMethod method) {
		this.method = method;
	}

	public boolean isIf() {
		return ifClause;
	}

	/**
	 * If false, this tab will not be rendered. Default true.
	 */
	@Attribute
	public void setIf(boolean ifClause) {
		this.ifClause = ifClause;
	}
	
}
