/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.util;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.loom.exception.ConfigException;
import org.loom.exception.MethodNotFoundException;
import org.loom.exception.PropertyNotFoundException;
import org.loom.exception.WrappedException;
import org.loom.log.Log;
import org.springframework.beans.BeanUtils;
import org.springframework.core.BridgeMethodResolver;

import com.google.common.collect.Lists;


/**
 * Yet Another ClassUtils implementation.
 * @author icoloma
 */
public class ClassUtils {

	private static Log log = Log.getLog();
	
	/** 
	 * Gets the short (unqualified) name of a Class
	 * @deprecated use Class.getSimpleName() instead
	 */
	public static String getShortName(Class<?> clazz) {
		return clazz.getSimpleName(); // StringUtils.substringAfterLast(clazz.getName(), ".");
	}
	
	/** 
	 * Gets the short (unqualified) name of a Method, without the class name
	 */
	public static String getShortName(Method method) {
		String result = method.getName();
		int lastPos = result.lastIndexOf('.');
		return lastPos == -1? result : result.substring(lastPos + 1);
	}
	
	/**
	 * @return new instance of type clazz
	 */
	public static <T> T newInstance(Class<T> clazz) {
		try {
			return clazz.newInstance();
		} catch (InstantiationException e) {
			throw new WrappedException(e);
		} catch (IllegalAccessException e) {
			throw new WrappedException(e);
		}
	}
	public static Object newInstance(String classname) {
		try {
			return newInstance(Class.forName(classname));
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Invokes a method, but all the throwable exceptions are translated to Runtime
	 */
	public static Object invoke(Object container, Method method, Object... args) {
		try {
			return method.invoke(container, args);
		} catch (IllegalAccessException e) {
			throw new RuntimeException("Could not call method: " + getSignature(method, args), e);
		} catch (InvocationTargetException e) {
			throw new WrappedException(e.getTargetException());
		}
	}
	
	/**
	 * @return the signature of a given method with the given params
	 */
	public static String getSignature(Method method, Object[] args) {
		return method.getDeclaringClass().getName() + "." + method.getName() + "(" + StringUtils.join(args, ", ") + ")";
	}

	/**
	 * Sets a field value, but all the throwable exceptions are translated to Runtime
	 */
	public static void setFieldValue(Object container, Field field, Object value) {
		try {
			field.set(container, value);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Gets a field value, but all the throwable exceptions are translated to Runtime
	 */
	public static Object getFieldValue(Object container, Field field) {
		try {
			return field.get(container);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @return the introspection PropertyDescriptor object for the required property, null if
	 * there is none
	 */
	public static PropertyDescriptor getPropertyDescriptor(Class<?> containerClass, String propertyName) {
		return BeanUtils.getPropertyDescriptor(containerClass, propertyName); //Introspector.getBeanInfo(containerClass);
	}

	/**
	 * @return the method of class clazz that is getter for property, null if none
	 */
	public static Method getGetter(Class<?> clazz, String property) {
		PropertyDescriptor descriptor = getPropertyDescriptor(clazz, property);
		Method method = descriptor == null? null : descriptor.getReadMethod();
		if (method !=  null) {
			method = BridgeMethodResolver.findBridgedMethod(method); 
		}
		return method;
	}
	
	/**
	 * @return the method of class clazz that is setter for property, null if none
	 */
	public static Method getSetter(Class<?> clazz, String property) {
		PropertyDescriptor descriptor = getPropertyDescriptor(clazz, property);
		Method method = descriptor == null? null : descriptor.getWriteMethod();
		if (method !=  null) {
			method = BridgeMethodResolver.findBridgedMethod(method); 
		}
		return method;
	}

	/**
	 * @return the required field, null if none. Looks in the parent classes. Searches also private Fields.
	 */
	public static Field getField(Class<?> clazz, String fieldName) {
		
		try {
			return clazz.getDeclaredField(fieldName);
		} catch (SecurityException e) {
			throw new PropertyNotFoundException(e);
		} catch (NoSuchFieldException e) {
			return findField(clazz, fieldName);
		}
	}
	
	/**
	 * @return searches the list of Field for that concrete Field
	 */
	private static Field findField(Class<?> clazz, String fieldName) {
		Field[] fields = clazz.getDeclaredFields();
		for (int i = 0; i < fields.length; i++) {
			Field field = fields[i];
			if (fieldName.equals(field.getName())) {
				return field;
			}
		}
		Class<?> parentClass = clazz.getSuperclass();
		if (parentClass != null && !parentClass.equals(Object.class)) {
			return findField(parentClass, fieldName);
		}
		return null;
	}

	/**
	 * @return the required method, null if none. Looks in the parent classes
	 */
	public static Method getMethod(Class<?> clazz, String methodName, Class<?>... types) {
		
		try {
			return clazz.getMethod(methodName, types);
		} catch (SecurityException e) {
			throw new MethodNotFoundException(e);
		} catch (NoSuchMethodException e) {
			Method m = findMethod(clazz.getSuperclass(), methodName, types);
			return m;
		}
	}
	
	/**
	 * @return a reasonable guess for the required method with these arguments
	 */
	public static Method findMethod(Class<?> clazz, String methodName, Class<?>... arguments) {
			int expectedArgCount = arguments == null? 0 : arguments.length;

			Method[] methods = clazz.getMethods();

			for (int i = 0; i < methods.length; i++) {
				Method result = methods[i];
				Class<?>[] paramTypes = result.getParameterTypes();
				int actualArgCount = paramTypes.length;
				if (result.getName().equals(methodName) && actualArgCount == expectedArgCount) {
					boolean match = true;
					for (int j = 0; j < actualArgCount && match; j++) {
						match = match && isAssignable(paramTypes[j], arguments[j]);
					}
					if (match) {
						return result;
					}
				}
			}
			return null;
	}
	
	/**
	 * @return a list of all methods that comply with a certain signature, ordered from Child to parent
	 */
	public static Method[] findMethodsReturning(Class<?> clazz, Class<?> returnType) {
		Method[] candidates = clazz.getMethods();
		List<Method> methods = Lists.newArrayListWithCapacity(candidates.length);  
		for (int i = 0; i < candidates.length; i++) {
			Method candidate = candidates[i];
			if (isAssignable(returnType, candidate.getReturnType()) && Modifier.isPublic(candidate.getModifiers())) {
				methods.add(candidate);
			}
		}
		return methods.toArray(new Method[methods.size()]);
	}
	
	/**
	 * @return a list of all methods that comply with a certain signature, ordered from Child to parent
	 */
	public static Method[] findMethods(Class<?> clazz, Class<?> returnType, Class<?>... arguments) {
		ArrayList<Method> methods = new ArrayList<Method>();
		internalFindMethods(methods, clazz, returnType, arguments);
		return methods.toArray(new Method[methods.size()]);
	}
	private static void internalFindMethods(List<Method> matchingMethods, Class<?> clazz, Class<?> returnType, Class<?>[] arguments) {
		int expectedArgCount = arguments == null? 0 : arguments.length;
		
		Method[] methods = clazz.getMethods();
		
		for (int i = 0; i < methods.length; i++) {
			Method candidate = methods[i];
			Class<?>[] paramTypes = candidate.getParameterTypes();
			int actualArgCount = paramTypes.length;
			if (actualArgCount == expectedArgCount && isAssignable(returnType, candidate.getReturnType()) && Modifier.isPublic(candidate.getModifiers())) {
				boolean match = true;
				for (int j = 0; j < actualArgCount && match; j++) {
					match = match && isAssignable(paramTypes[j], arguments[j]);
				}
				if (match) {
					matchingMethods.add(candidate);
				}
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	public static boolean isAssignable(Class targetType, Class valueType) {
		return targetType.isAssignableFrom(valueType);
	}

	/**
	 * @return true if the class can be instantiated (is not abstract or interface)
	 */
	public static boolean isAbstract(Class<?> clazz) {
		return !clazz.isPrimitive() && (Modifier.isAbstract(clazz.getModifiers()) || clazz.isInterface());
	}

	public static Class<?> forName(String className) {
		try {
			return Class.forName(className);
		} catch (ClassNotFoundException e) {
			throw new ConfigException(e);
		}
	}

	/**
	 * If the provided method is a getter or a setter, returns the lowercased property name, ex
	 * beanName for getBeanName 
	 * @param method
	 * @return the property name, null if the method is not a getter or setter
	 */
	public static String getPropertyName(Method method) {
		String methodName = method.getName();
		if (methodName.length() < 4) {
			return null;
		}
		Class<?>[] parameters = method.getParameterTypes();
		boolean returnsNull = Void.TYPE.equals(method.getReturnType());
		if (
			(methodName.startsWith("get") && parameters.length == 0 && !returnsNull) ||
			(methodName.startsWith("set") && parameters.length == 1 && returnsNull)) {
			return Character.toLowerCase(methodName.charAt(3)) + methodName.substring(4); 
		}
		return null;
			
	}

	/**
	 * @return true if the provided classname is in the classpath
	 */
	public static boolean isPresent(String className) {
		try {
			Class.forName(className);
			return true;
		} catch (NoClassDefFoundError e) {
			log.debug("No class " + className + " found in the classpath");
			return false;
		} catch (ClassNotFoundException e) {
			log.debug("No class " + className + " found in the classpath");
			return false;
		}
	}

}
