/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.util;

import org.apache.commons.lang.StringUtils;
import org.loom.exception.MalformedPropertyPathException;


public class PropertyUtils {

	public static String[] split(String propertyPath) {
		return StringUtils.split(propertyPath, '.');
	}
	
	/**
	 * Normalize a property path. Normalized property paths are the only way to retrieve the correct
	 * PropertyBinder from the Map
	 * Removes all indexes from a propertyPath
	 * @param propertyPath the property path as is in the HttpServletRequest object, e.g. child.parent[5].name
	 * @return the propertyPath without indexes, e.g. child.parent.name
	 * @throws MalformedPropertyPathException If there is any index that misses the ending ']'
	 */
	public static String normalize(String propertyPath) {
		// For the record: this implementation is six times faster
		// than using regex
		int length = propertyPath.length();
		StringBuilder result = new StringBuilder(length);
		int init = 0;
		while (init < length) {
			int end = propertyPath.indexOf('[', init);
			if (end == -1) {
				end = length;
			}
			result.append(propertyPath.substring(init, end));
			if (end < length) {
				init = propertyPath.indexOf(']', end + 1);
				if (init == -1) {
					throw new MalformedPropertyPathException("Missing ']' at " + propertyPath);
				}
				init = init + 1;
			} else {
				break;
			} 
		} 
		return result.toString();
	}

	/**
	 * Concats properties to form a property path
	 */
	public static String concatPropertyPath(String... propertyNames) {
		StringBuilder builder = new StringBuilder(20);
		for (String propertyName : propertyNames) {
			if (!StringUtils.isEmpty(propertyName)) {
				if (builder.length() > 0) {
					builder.append(".");
				}
				builder.append(propertyName);
			}
		}
		return builder.toString();
	}

	/**
	 * Pops the root level of a property path:
	 * <ul><li>a.b.c becomes b.c</li>
	 * <li>a.b becomes b</li>
	 * <li>a becomes null</li></ul>
	 */
	public static String removeRootNode(String propertyPath) {
		int index = propertyPath.indexOf('.');
		return index == -1 || index == propertyPath.length() - 1? null : propertyPath.substring(index + 1);
	}
	
	/**
	 * Returns the last node of a property path
	 * <ul><li>a.b.c becomes c</li>
	 * <li>a.b becomes b</li>
	 * <li>a becomes a</li></ul>
	 */
	public static String getLastNode(String propertyPath) {
		int index = propertyPath.lastIndexOf('.');
		return index == -1? propertyPath : propertyPath.substring(index + 1);
	}
	

	
}
