/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.validator;

import java.util.Iterator;
import java.util.List;

import org.loom.i18n.Message;
import org.loom.i18n.Messages;

import com.google.common.collect.Lists;


/**
 * Class with all the data needed to perform a validation
 * @author icoloma
 */
public class ValidationRequest {

	/** the validator to be executed */
	private PropertyValidator validator;
	
	/** any validation error message should go here */
	private Messages messages;

	private List<VisitedNode> visitedPath = Lists.newArrayList(); 
	
	public ValidationRequest(PropertyValidator validator, Messages messages) {
		this.validator = validator;
		this.messages = messages;
	}
	
	/**
	 * Performs the validation
	 * @param value the value to validate
	 */
	public void validate(Object value) {
		validator.validate(this, value);
	}

	public Messages getMessages() {
		return messages;
	}

	public PropertyValidator getValidator() {
		return validator;
	}
	
	public ValidationRequest push(String nodeName) {
		push(nodeName, null);
		return this;
	}
	public ValidationRequest push(String nodeName, Object index) {
		visitedPath.add(new VisitedNode(nodeName, index));
		return this;
	}
	public void pop() {
		visitedPath.remove(visitedPath.size() - 1);
	}
	
	/**
	 * Adds a validation error message
	 * @param messageKey The key of the error message
	 * @param value The value that could not be validated
	 * @return the Message that has been added, or null if the field already had associated errors.
	 */
	public Message addErrorMessage(String messageKey, Object value) {
		String visitedPath = getVisitedPath();
		// omit validation errors for fields which already have conversion errors
		if (messages.hasAnyError(visitedPath)) {
			return null;
		}
		
		if (validator.getOverridenMessageKey() != null) {
			messageKey = validator.getOverridenMessageKey();
		}
		Message message = new Message(messageKey, validator.isBindMessages()? visitedPath :null);
		message.addArg(Message.VALUE_ARG, value);
		message.addArg("validator", validator);
		message.addTranslatedArg(Message.PROPERTY_PATH_ARG, validator.getPropertyPath());
		messages.addMessage(message);
		return message;
	}
	
	/**
	 * @return a String that represents the visited property path with property indexes inclued, if any
	 */
	@SuppressWarnings("unchecked")
	public String getVisitedPath() {
		StringBuilder result = new StringBuilder();
		for (Iterator<VisitedNode> i = visitedPath.iterator(); i.hasNext(); ) {
			i.next().addTo(result);
		}
		return result.toString();
	}

	/**
	 * Visited nodes when validating
	 */
	private class VisitedNode {
		
		/** name of the node */
		private String name;
		
		/** node index, if any (Collections, Maps, arrays) */
		private Object index;

		public VisitedNode(String name, Object index) {
			this.name = name;
			this.index = index;
		}
		
		/**
		 * Appends this node to the builer passed as argument
		 */
		public void addTo(StringBuilder builder) {
			if (builder.length() > 0) {
				builder.append('.');
			}
			builder.append(name);
			if (index != null) {
				builder.append('[').append(index).append(']');
			}
		}
		
		// for debugging purposes
		@Override
		public String toString() {
			StringBuilder b = new StringBuilder();
			addTo(b);
			return b.toString();
		}
		
	}

	
	
}
